package org.bidib.wizard.locale;

import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Helper class for localization.
 * 
 * @author Andr&eacute; Schenk
 * @version $Revision: 1.2 $
 */
public final class Resources {
    private static Resources resources = new Resources();

    private ResourceBundle properties = null;

    private static final int START_POS = "org.bidib.wizard.".length();

    private Resources() {
        properties = ResourceBundle.getBundle("catalogs/BiDiBWizard_locale");
    }

    private static synchronized Resources getInstance() {
        return resources;
    }

    private static ResourceBundle getResources() {
        return getInstance().properties;
    }

    public static String getString(Class<?> clazz, String key) {
        String result = null;
        ResourceBundle properties = getResources();
        String resourceName = clazz.getName().substring(START_POS) + "." + key;

        if (properties != null) {
            try {
                result = properties.getString(resourceName);
            }
            catch (MissingResourceException e) {
                result = "<" + resourceName + ">";
            }
        }
        else {
            result = resourceName;
        }
        return result;
    }

    public static String getString(Class<?> clazz, String key, Object[] arguments) {
        String result = null;
        String template = getString(clazz, key);

        if (template != null) {
            result = new MessageFormat(template).format(arguments);
        }
        return result;
    }
}
