package org.bidib.wizard.main;

import gnu.io.PortInUseException;
import gnu.io.UnsupportedCommOperationException;

import java.io.File;
import java.io.IOException;

import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.bidib.jbidibc.Bidib;
import org.bidib.jbidibc.exception.PortNotFoundException;
import org.bidib.jbidibc.exception.ProtocolException;
import org.bidib.wizard.mvc.error.controller.ErrorController;
import org.bidib.wizard.mvc.main.controller.MainController;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.slf4j.LoggerFactory;

public final class BiDiBWizard {
    private BiDiBWizard() {
    }

    public static void main(String[] args) throws ClassNotFoundException, InstantiationException,
        IllegalAccessException, UnsupportedLookAndFeelException, PortNotFoundException, PortInUseException,
        UnsupportedCommOperationException, IOException, ProtocolException {
        // set path to preferences
        Preferences.setPath(new File(System.getProperty("user.home"), ".bidibwizardrc").getPath());

        // set system look and feel only under windows
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

        Preferences.getInstance().reloadLoggerConfiguration(true);

        // set log file for analyzing
        if (args.length >= 1) {
            LoggerFactory.getLogger(BiDiBWizard.class).info("Set logfile: {}", args[0]);
            Bidib.getInstance().setLogFile(args[0]);
        }

        // open GUI
        new ErrorController().start();
        new MainController().start();
    }
}
