package org.bidib.wizard.mvc.cvprogrammer.view;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.cvprogrammer.model.CvProgrammerModel;
import org.bidib.wizard.mvc.cvprogrammer.view.listener.CvProgrammerViewListener;
import org.bidib.wizard.mvc.cvprogrammer.view.panel.CvReadPanel;
import org.bidib.wizard.mvc.cvprogrammer.view.panel.CvWritePanel;
import org.bidib.wizard.mvc.cvprogrammer.view.panel.listener.CvReadListener;
import org.bidib.wizard.mvc.cvprogrammer.view.panel.listener.CvWriteListener;

public class CvProgrammerView extends JDialog {
    private static final long serialVersionUID = -6895526911794092393L;

    private final Collection<CvProgrammerViewListener> listeners = new LinkedList<CvProgrammerViewListener>();

    private final CvProgrammerModel model;

    private final CvReadPanel cvReadPanel;

    private final CvWritePanel cvWritePanel;

    private final JButton closeButton = new JButton(Resources.getString(getClass(), "close"));

    public CvProgrammerView(JFrame parent, CvProgrammerModel model, int x, int y) {
        super(parent);
        this.model = model;
        cvReadPanel = new CvReadPanel(model);
        cvWritePanel = new CvWritePanel(model);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setLocation(x, y);
        // setModal(true);
        setResizable(true);
        setTitle(Resources.getString(getClass(), "title"));

        JPanel mainPanel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 1;

        cvReadPanel.addCvReadListener(new CvReadListener() {
            @Override
            public void read(int cvNumber) {
                fireRead(cvNumber);
            }
        });
        mainPanel.add(cvReadPanel, c);

        cvWritePanel.addCvWriteListener(new CvWriteListener() {
            @Override
            public void write(int cvValue) {
                fireWrite(cvValue);
            }
        });
        c.gridy++;
        mainPanel.add(cvWritePanel, c);

        closeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                close();
            }
        });
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.fill = GridBagConstraints.NONE;
        c.gridy++;
        c.weighty = 1;
        mainPanel.add(closeButton, c);

        getContentPane().add(mainPanel);
        pack();
        setMinimumSize(getSize());
        setSize(new Dimension(getSize().width + 10, getSize().height));

        int labelWidth = Math.max(cvReadPanel.getLabelWidth(), cvWritePanel.getLabelWidth());

        cvReadPanel.setLabelWidth(labelWidth);
        cvWritePanel.setLabelWidth(labelWidth);

        int buttonWidth =
            Math.max(Math.max(cvReadPanel.getButtonWidth(), cvWritePanel.getButtonWidth()), closeButton.getWidth());

        cvReadPanel.setButtonWidth(buttonWidth);
        cvWritePanel.setButtonWidth(buttonWidth);
        closeButton.setPreferredSize(new Dimension(buttonWidth, closeButton.getSize().height));
        setVisible(true);
    }

    public void addCvProgrammerViewListener(CvProgrammerViewListener l) {
        listeners.add(l);
    }

    private void close() {
        setVisible(false);
        fireClose();
    }

    private void fireClose() {
        for (CvProgrammerViewListener l : listeners) {
            l.close();
        }
    }

    private void fireRead(int cvNumber) {
        for (CvProgrammerViewListener l : listeners) {
            l.read(cvNumber);
        }
    }

    private void fireWrite(int cvValue) {
        for (CvProgrammerViewListener l : listeners) {
            l.write(model.getNumber(), cvValue);
        }
    }
}
