package org.bidib.wizard.mvc.features.controller;

import java.util.Collection;
import java.util.HashSet;

import javax.swing.JFrame;

import org.bidib.jbidibc.Feature;
import org.bidib.wizard.mvc.features.controller.listener.FeaturesControllerListener;
import org.bidib.wizard.mvc.features.model.FeaturesModel;
import org.bidib.wizard.mvc.features.view.FeaturesView;
import org.bidib.wizard.mvc.features.view.listener.FeaturesViewListener;
import org.bidib.wizard.mvc.main.model.Node;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FeaturesController {
    private static final Logger LOGGER = LoggerFactory.getLogger(FeaturesController.class);

    private final Collection<FeaturesControllerListener> listeners = new HashSet<FeaturesControllerListener>();

    private final JFrame parent;

    private final Node node;

    private final int x;

    private final int y;

    private final FeaturesModel model = new FeaturesModel();

    private FeaturesView view;

    public FeaturesController(JFrame parent, Node node, int x, int y) {
        this.parent = parent;
        this.node = node;
        this.x = x;
        this.y = y;
    }

    public void addFeaturesControllerListener(FeaturesControllerListener listener) {
        listeners.add(listener);
    }

    private void fireClose() {
        for (FeaturesControllerListener listener : listeners) {
            listener.close();
        }
    }

    private void fireReadAll() {
        LOGGER.debug("Read all features.");
        for (FeaturesControllerListener listener : listeners) {
            listener.readAll(node);
        }
    }

    private void fireWrite(Feature feature) {
        LOGGER.debug("Write feature: {}", feature);
        for (FeaturesControllerListener listener : listeners) {
            listener.write(node, feature);
        }
    }

    public void start() {
        view = new FeaturesView(parent, model, x, y);
        view.addFeaturesViewListener(new FeaturesViewListener() {
            @Override
            public void close() {
                fireClose();
            }

            @Override
            public void readAll() {
                fireReadAll();
            }

            @Override
            public void write(Collection<Feature> features) {
                // iterate over the features and write the values
                for (Feature feature : features) {
                    // TODO what happens if an error occurs?
                    fireWrite(feature);
                }

                // TODO should update the currently displayed node panels ...
            }
        });

        // initially read all features
        fireReadAll();

        // make the view visible
        view.setVisible(true);
    }

    public void setFeatures(Collection<Feature> features) {
        LOGGER.debug("Set the features in the model.");

        model.setFeatures(features);
    }

}
