package org.bidib.wizard.mvc.features.view;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.HashSet;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import org.bidib.jbidibc.Feature;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.features.model.FeaturesModel;
import org.bidib.wizard.mvc.features.view.listener.FeaturesViewListener;
import org.bidib.wizard.mvc.features.view.panel.FeaturesPanel;
import org.bidib.wizard.mvc.features.view.panel.listener.FeaturesWriteListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FeaturesView extends JDialog {
    private static final Logger LOGGER = LoggerFactory.getLogger(FeaturesView.class);

    private static final long serialVersionUID = 1L;

    //    private final FeaturesModel model;
    private final Collection<FeaturesViewListener> listeners = new HashSet<FeaturesViewListener>();

    private final JButton saveButton = new JButton(Resources.getString(getClass(), "save"));

    private final JButton cancelButton = new JButton(Resources.getString(getClass(), "cancel"));

    private final FeaturesPanel featuresPanel;

    public FeaturesView(JFrame parent, FeaturesModel model, int x, int y) {
        super(parent);
        //        this.model = model;

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                cancel();
            }
        });
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setLocation(x, y);
        setResizable(true);
        setTitle(Resources.getString(getClass(), "title"));

        JPanel mainPanel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 1;
        c.weighty = 1;

        featuresPanel = new FeaturesPanel(model);
        JScrollPane scroll = new JScrollPane();
        scroll.setViewportView(featuresPanel.getTable());
        mainPanel.add(scroll, c);

        featuresPanel.addFeaturesWriteListener(new FeaturesWriteListener() {
            @Override
            public void write(Collection<Feature> features) {
                fireWrite(features);
            }
        });

        // buttons
        GridLayout gl = new GridLayout(0, 2, 5, 0);
        JPanel buttons = new JPanel(gl);

        saveButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                save();
            }
        });
        buttons.add(saveButton);

        cancelButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                cancel();
            }
        });
        buttons.add(cancelButton);

        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.fill = GridBagConstraints.NONE;
        c.gridx = 0;
        c.gridy++;
        mainPanel.add(buttons, c);

        getContentPane().add(mainPanel);
        pack();
        setMinimumSize(getSize());
        setSize(new Dimension(getSize().width + 10, getSize().height));
        setPreferredSize(new Dimension(400, 200));

        int buttonWidth =
            Math.max(/*Math.max(cvReadPanel.getButtonWidth(), cvWritePanel.getButtonWidth())*/saveButton.getWidth(),
                cancelButton.getWidth());

        //        cvReadPanel.setButtonWidth(buttonWidth);
        //        cvWritePanel.setButtonWidth(buttonWidth);
        saveButton.setPreferredSize(new Dimension(buttonWidth, saveButton.getSize().height));
        cancelButton.setPreferredSize(new Dimension(buttonWidth, cancelButton.getSize().height));
        //        setVisible(true);
    }

    public void addFeaturesViewListener(FeaturesViewListener listener) {
        listeners.add(listener);
    }

    private void save() {
        LOGGER.debug("Save the changed features.");

        // write the features to the node
        featuresPanel.writeFeatures();

        // close the dialog
        setVisible(false);
        fireClose();
    }

    private void cancel() {
        setVisible(false);
        fireClose();
    }

    private void fireClose() {
        for (FeaturesViewListener listener : listeners) {
            listener.close();
        }
    }

    private void fireWrite(Collection<Feature> features) {
        for (FeaturesViewListener listener : listeners) {
            listener.write(features);
        }
    }
}
