package org.bidib.wizard.mvc.features.view.panel;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import org.bidib.jbidibc.Feature;
import org.bidib.wizard.locale.Resources;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FeatureTableModel extends AbstractTableModel {
    private static final Logger LOGGER = LoggerFactory.getLogger(FeatureTableModel.class);

    private static final long serialVersionUID = 1L;

    private static final String[] _COLUMN_NAMES_ =
        { Resources.getString(FeaturesPanel.class, "feature"), Resources.getString(FeaturesPanel.class, "value") };

    private List<Feature> features = new LinkedList<Feature>();

    public FeatureTableModel() {
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        if (column == 1) {
            return true;
        }
        return false;
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        if (columnIndex == 1) {
            return Integer.class;
        }
        return super.getColumnClass(columnIndex);
    }

    public void updateRow(Feature feature) {

        // check if the feature is already displayed, if yes: update the row, otherwise: add new row
        if (features.contains(feature)) {
            LOGGER.debug("Feature is already in list and will be removed: {}", feature);
            features.remove(feature);
        }
        // create a copy of the feature to store in the feature list
        Feature copy = new Feature(feature.getType(), feature.getValue());
        features.add(copy);

        fireTableDataChanged();
    }

    @Override
    public int getColumnCount() {
        return _COLUMN_NAMES_.length;
    }

    @Override
    public String getColumnName(int column) {
        return _COLUMN_NAMES_[column];
    }

    @Override
    public int getRowCount() {
        return features.size();
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Feature feature = features.get(rowIndex);
        switch (columnIndex) {
            case 0:
                return feature.getFeatureName();
            default:
                return feature.getValue();
        }
    }

    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex) {
        Feature feature = features.get(rowIndex);
        switch (columnIndex) {
            case 1:
                if (value instanceof Integer) {
                    feature.setValue(((Integer) value).intValue());
                    break;
                }
            default:
                break;
        }
    }

    public Feature getRowAt(int rowIndex) {
        Feature feature = features.get(rowIndex);
        return feature;
    }

    public void clear() {
        features.clear();
        fireTableDataChanged();
    }

    public List<Feature> getFeatures() {
        return Collections.unmodifiableList(features);
    }
}
