package org.bidib.wizard.mvc.firmware.view;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.firmware.model.FirmwareModel;
import org.bidib.wizard.mvc.firmware.model.MemoryType;
import org.bidib.wizard.mvc.firmware.model.listener.FirmwareModelListener;
import org.bidib.wizard.mvc.firmware.view.listener.FirmwareViewListener;
import org.bidib.wizard.mvc.firmware.view.panel.FirmwareChoosePanel;
import org.bidib.wizard.mvc.firmware.view.panel.FirmwareInfoPanel;
import org.bidib.wizard.mvc.firmware.view.panel.FirmwareProcessPanel;
import org.bidib.wizard.mvc.firmware.view.panel.listener.FirmwareChooseListener;
import org.bidib.wizard.mvc.firmware.view.panel.listener.FirmwareProcessListener;

public class FirmwareView extends JDialog {
    private static final long serialVersionUID = 3656752143993796411L;

    private final Collection<FirmwareViewListener> listeners = new LinkedList<FirmwareViewListener>();

    private final JButton closeButton = new JButton(Resources.getString(getClass(), "close"));

    public FirmwareView(JFrame parent, final FirmwareModel model, int x, int y) {
        super(parent);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                if (!model.isInProgress()) {
                    setVisible(false);
                    fireClose();
                }
            }
        });
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setLocation(x, y);
        setResizable(true);
        setTitle(Resources.getString(getClass(), "title"));

        JPanel mainPanel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 1;

        FirmwareInfoPanel firmwareInfoPanel = new FirmwareInfoPanel(model);

        mainPanel.add(firmwareInfoPanel, c);

        FirmwareChoosePanel firmwareChoosePanel = new FirmwareChoosePanel();

        firmwareChoosePanel.addFirmwareChooseListener(new FirmwareChooseListener() {
            @Override
            public void chooseFirmware(File file, MemoryType type) {
                fireChooseFirmware(file, type);
            }
        });
        c.gridy++;
        mainPanel.add(firmwareChoosePanel, c);

        FirmwareProcessPanel firmwareProcessPanel = new FirmwareProcessPanel(model);

        firmwareProcessPanel.addFirmwareProcessListener(new FirmwareProcessListener() {
            @Override
            public void update() {
                model.setInProgress(true);
                fireUpdate();
            }
        });
        c.gridy++;
        mainPanel.add(firmwareProcessPanel, c);

        closeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (!model.isInProgress()) {
                    setVisible(false);
                    fireClose();
                }
            }
        });
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.fill = GridBagConstraints.NONE;
        c.gridy++;
        c.weightx = 0;
        c.weighty = 1;
        mainPanel.add(closeButton, c);

        getContentPane().add(mainPanel);
        pack();
        setMinimumSize(getSize());
        setSize(new Dimension(getSize().width + 10, getSize().height));

        int buttonWidth = Math.max(firmwareChoosePanel.getButtonWidth(), firmwareProcessPanel.getButtonWidth());

        firmwareInfoPanel.setButtonWidth(buttonWidth);
        firmwareChoosePanel.setButtonWidth(buttonWidth);
        firmwareProcessPanel.setButtonWidth(buttonWidth);

        int labelWith = Math.max(firmwareInfoPanel.getLabelWidth(), firmwareChoosePanel.getLabelWidth());

        firmwareInfoPanel.setLabelWidth(labelWith);
        firmwareProcessPanel.setLabelWidth(labelWith);

        model.addFirmwareModelListener(new FirmwareModelListener() {
            @Override
            public void inProgressChanged(final boolean inProgress) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        closeButton.setEnabled(!inProgress);
                    }
                });
            }

            @Override
            public void progressValueChanged(int progressValue) {
            }
        });

        setVisible(true);
    }

    public void addFirmwareViewListener(FirmwareViewListener l) {
        listeners.add(l);
    }

    private void fireChooseFirmware(File file, MemoryType type) {
        for (FirmwareViewListener l : listeners) {
            l.chooseFirmware(file, type);
        }
    }

    private void fireClose() {
        for (FirmwareViewListener l : listeners) {
            l.close();
        }
    }

    private void fireUpdate() {
        for (FirmwareViewListener l : listeners) {
            l.update();
        }
    }
}
