package org.bidib.wizard.mvc.main.model;

import java.util.Collection;
import java.util.LinkedList;

import javax.swing.table.DefaultTableModel;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.listener.AccessoryListListener;
import org.bidib.wizard.mvc.main.model.listener.AccessoryListener;
import org.bidib.wizard.mvc.main.model.listener.AccessoryPortListener;

public class AccessoryTableModel extends DefaultTableModel implements AccessoryListener {
    private static final long serialVersionUID = -246667717914497053L;

    private static final String[] COLUMN_NAMES =
        new String[] { Resources.getString(AccessoryTableModel.class, "aspect"),
            Resources.getString(AccessoryTableModel.class, "macro"),
            Resources.getString(AccessoryTableModel.class, "test") };

    private final Collection<AccessoryPortListener> portListeners = new LinkedList<AccessoryPortListener>();

    private final MainModel model;

    private Accessory accessory = null;

    public AccessoryTableModel(final MainModel model) {
        this.model = model;
        setColumnIdentifiers(COLUMN_NAMES);
        model.addAccessoryListListener(new AccessoryListListener() {
            @Override
            public void accessoryChanged() {
                if (accessory != null) {
                    accessory.removeAccessoryListener(AccessoryTableModel.this);
                }

                final Accessory accessory = model.getAccessory();

                if (accessory != null) {
                    accessory.addAccessoryListener(AccessoryTableModel.this);
                }
                AccessoryTableModel.this.accessory = accessory;
                macrosChanged();
            }

            @Override
            public void listChanged() {
            }
        });
    }

    public void addPortListener(AccessoryPortListener l) {
        portListeners.add(l);
    }

    private void addRow(int row, Macro macro) {
        Object[] rowData = new Object[COLUMN_NAMES.length];

        for (int column = 0; column < COLUMN_NAMES.length; column++) {
            rowData[column] = getRowData(row, column, macro);
        }
        addRow(rowData);
    }

    private void fireTestButtonPressed(int aspect) {
        for (AccessoryPortListener l : portListeners) {
            l.testButtonPressed(aspect);
        }
    }

    private Object getRowData(int row, int column, Macro macro) {
        Object result = null;

        switch (column) {
            case 0:
                result = Resources.getString(getClass(), "aspect") + "_" + row;
                break;
            case 1:
                result = macro;
                break;
            case 2:
                result = accessory;
                break;
        }
        return result;
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        return column > 0;
    }

    @Override
    public void setValueAt(Object value, int row, int column) {
        switch (column) {
            case 1:
                model.getAccessory().setMacro(row, ((Macro) value).getId());
                super.setValueAt(value, row, column);
                break;
            case 2:
                fireTestButtonPressed(row);
                break;
            default:
                super.setValueAt(value, row, column);
        }
    }

    @Override
    public void labelChanged(String label) {
    }

    @Override
    public void macrosChanged() {
        setRowCount(0);
        for (int row = 0; row < accessory.getMacros().size(); row++) {
            addRow(row, model.getMacros().get(accessory.getMacros().get(row)));
        }
    }
}
