package org.bidib.wizard.mvc.main.model;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.bidib.wizard.mvc.main.model.listener.FeedbackPortListener;

public class FeedbackPortTableModel extends SimplePortTableModel {
    private static final long serialVersionUID = 1190327867251252042L;

    private static final int COLUMNS = 8;

    // map from port to table position
    private final Map<Port, Integer> portIndexes = new HashMap<Port, Integer>();

    public FeedbackPortTableModel(MainModel model) {
        setColumnIdentifiers(new String[COLUMNS + 1]);
        model.addFeedbackPortListener(new FeedbackPortListener() {
            @Override
            public void addressesChanged(Port port, Collection<AddressData> addresses) {
                int index = portIndexes.get(port);
                int row = index / COLUMNS;
                int column = index % COLUMNS;

                setValueAt(port, row, column);
            }

            @Override
            public void confidenceChanged(Port port, boolean freeze, boolean signal, boolean valid) {
                int index = portIndexes.get(port);
                int row = index / COLUMNS;
                int column = index % COLUMNS;

                setValueAt(port, row, column);
            }

            @Override
            public void labelChanged(Port port, String label) {
            }

            @Override
            public void speedChanged(Port port, int address, int speed) {
                int index = portIndexes.get(port);
                int row = index / COLUMNS;
                int column = index % COLUMNS;

                setValueAt(port, row, column);
            }

            @Override
            public void statusChanged(Port port, Enum<?> status) {
                int index = portIndexes.get(port);
                int row = index / COLUMNS;
                int column = index % COLUMNS;

                setValueAt(port, row, column);
            }
        });
    }

    public void addRow(Collection<? extends Port> ports) {
        if (ports != null) {
            List<Object> rowData = new LinkedList<Object>();
            int index = 0;

            for (Port port : ports) {
                portIndexes.put(port, index++);
                rowData.add(port);
                if (index > 0 && index % COLUMNS == 0) {
                    // dummy column which will be removed from SimplePortListPanel
                    rowData.add("");
                    addRow(rowData.toArray());
                    rowData.clear();
                }
            }
        }
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        return false;
    }
}
