package org.bidib.wizard.mvc.main.model;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;

public abstract class Labels {
    private static final Logger LOGGER = LoggerFactory.getLogger(Labels.class);

    protected Object labelMap = null;

    /**
     * Flag to signal if the data is modified.
     */
    protected transient boolean modified = false;

    private String getFileName() {
        String result = null;
        File baseDir = new File(Preferences.getInstance().getLabelPath());

        if (!baseDir.exists()) {
            baseDir.mkdir();
        }
        if (baseDir.exists()) {
            result = new File(baseDir, getClass().getSimpleName()).toString();
        }
        return result;
    }

    public Object getLabelMap() {
        return labelMap;
    }

    public void load() {
        XMLDecoder d = null;

        try {
            d = new XMLDecoder(new InputSource(XmlLoader.changePackage(new FileReader(getFileName()))));

            Labels labels = (Labels) d.readObject();

            if (labels != null) {
                setLabelMap(labels.getLabelMap());
            }
        }
        catch (FileNotFoundException e) {
            LOGGER.trace("Read labels from file failed because the file is not available.", e);
        }
        finally {
            if (d != null) {
                d.close();
            }
        }
    }

    public void save() throws IOException {
        if (modified) {
            XMLEncoder e = new XMLEncoder(new BufferedOutputStream(new FileOutputStream(getFileName())));

            e.writeObject(this);
            e.close();
            modified = false;
        }
    }

    public void setLabelMap(Object labelMap) {
        this.labelMap = labelMap;
    }
}
