package org.bidib.wizard.mvc.main.model;

import java.util.Collection;
import java.util.LinkedList;

import javax.swing.table.DefaultTableModel;

import org.bidib.wizard.comm.LightPortStatus;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.listener.LightPortListener;

public class LightPortTableModel extends DefaultTableModel {
    private static final long serialVersionUID = 3414290611732742790L;

    private final Collection<LightPortListener> portListeners = new LinkedList<LightPortListener>();

    private final String[] columnNames =
        { Resources.getString(getClass(), "label"), Resources.getString(getClass(), "pwmMin"),
            Resources.getString(getClass(), "pwmMax"), Resources.getString(getClass(), "dimMin"),
            Resources.getString(getClass(), "dimMax"), Resources.getString(getClass(), "test"), null };

    public LightPortTableModel() {
        setColumnIdentifiers(columnNames);
    }

    public void addLightPortListener(LightPortListener l) {
        portListeners.add(l);
    }

    public void addRow(LightPort port) {
        if (port != null) {
            Object[] rowData = new Object[columnNames.length];

            rowData[0] = port.toString();
            rowData[1] = port.getPwmMin();
            rowData[2] = port.getPwmMax();
            rowData[3] = port.getDimMin();
            rowData[4] = port.getDimMax();
            rowData[5] = port.getStatus();
            rowData[6] = port;
            addRow(rowData);
        }
    }

    private void fireLabelChanged(LightPort port, String label) {
        for (LightPortListener l : portListeners) {
            l.labelChanged(port, label);
        }
    }

    private void fireValuesChanged(LightPort port) {
        for (LightPortListener l : portListeners) {
            l.valuesChanged(port);
        }
    }

    @Override
    public Class<?> getColumnClass(int column) {
        switch (column) {
            case 0:
                return String.class;
            case 5:
                return Object.class;
            default:
                return Integer.class;
        }
    }

    @Override
    public void setValueAt(Object value, int row, int column) {
        final LightPort port = (LightPort) getValueAt(row, 6);

        switch (column) {
            case 0:
                port.setLabel((String) value);
                super.setValueAt(port.toString(), row, column);
                fireLabelChanged(port, port.getLabel());
                break;
            case 1:
                port.setPwmMin((Integer) value);
                super.setValueAt(value, row, column);
                fireValuesChanged(port);
                break;
            case 2:
                port.setPwmMax((Integer) value);
                super.setValueAt(value, row, column);
                fireValuesChanged(port);
                break;
            case 3:
                try {
                    port.setDimMin(Integer.parseInt(value.toString()));
                    super.setValueAt(value, row, column);
                    fireValuesChanged(port);
                }
                catch (NumberFormatException e) {
                }
                break;
            case 4:
                try {
                    port.setDimMax(Integer.parseInt(value.toString()));
                    super.setValueAt(value, row, column);
                    fireValuesChanged(port);
                }
                catch (NumberFormatException e) {
                }
                break;
            case 5:
                port.setStatus((LightPortStatus) value);
                super.setValueAt(value, row, column);
                break;
        }
    }
}
