package org.bidib.wizard.mvc.main.model;

import java.util.Collection;
import java.util.LinkedList;

import org.bidib.jbidibc.enumeration.IdentifyState;
import org.bidib.wizard.mvc.main.model.listener.NodeListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Node implements LabelAware {
    private static final Logger LOGGER = LoggerFactory.getLogger(Node.class);

    private final Collection<NodeListener> listeners = new LinkedList<NodeListener>();

    private Boolean addressMessagesEnabled = null;

    private Boolean commandStation = null;

    private Boolean dccStartEnabled = null;

    private Boolean externalStartEnabled = null;

    private Boolean feedbackMessagesEnabled = null;

    private IdentifyState identifyState = null;

    private Boolean keyMessagesEnabled = null;

    private String label = null;

    private org.bidib.jbidibc.Node node = null;

    private int storableMacroCount = 0;

    private boolean updatable = false;

    public void addNodeListener(NodeListener l) {
        listeners.add(l);
    }

    public void removeNodeListener(NodeListener l) {
        listeners.remove(l);
    }

    public Boolean isAddressMessagesEnabled() {
        return addressMessagesEnabled;
    }

    public void setAddressMessagesEnabled(Boolean addressMessagesEnabled) {
        this.addressMessagesEnabled = addressMessagesEnabled;
        fireAddressMessagesEnabledChanged(addressMessagesEnabled);
    }

    public Boolean isCommandStation() {
        return commandStation;
    }

    public void setCommandStation(Boolean commandStation) {
        this.commandStation = commandStation;
    }

    public Boolean isDccStartEnabled() {
        return dccStartEnabled;
    }

    public void setDccStartEnabled(Boolean dccStartEnabled) {
        this.dccStartEnabled = dccStartEnabled;
        fireDccStartEnabledChanged(dccStartEnabled);
    }

    public Boolean isExternalStartEnabled() {
        return externalStartEnabled;
    }

    public void setExternalStartEnabled(Boolean externalStartEnabled) {
        this.externalStartEnabled = externalStartEnabled;
    }

    public Boolean isFeedbackMessagesEnabled() {
        return feedbackMessagesEnabled;
    }

    public void setFeedbackMessagesEnabled(Boolean feedbackMessagesEnabled) {
        this.feedbackMessagesEnabled = feedbackMessagesEnabled;
        fireFeedbackMessagesEnabledChanged(feedbackMessagesEnabled);
    }

    public IdentifyState getIdentifyState() {
        return identifyState;
    }

    public void setIdentifyState(IdentifyState identifyState) {
        this.identifyState = identifyState;
        fireIdentifyStateChanged(identifyState);
    }

    public Boolean isKeyMessagesEnabled() {
        return keyMessagesEnabled;
    }

    public void setKeyMessagesEnabled(Boolean keyMessagesEnabled) {
        this.keyMessagesEnabled = keyMessagesEnabled;
        fireKeyMessagesEnabledChanged(keyMessagesEnabled);
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
        fireLabelChanged(label);
    }

    public org.bidib.jbidibc.Node getNode() {
        return node;
    }

    public void setNode(org.bidib.jbidibc.Node node) {
        this.node = node;
    }

    public int getStorableMacroCount() {
        return storableMacroCount;
    }

    public void setStorableMacroCount(int storableMacroCount) {
        this.storableMacroCount = storableMacroCount;
    }

    public boolean isUpdatable() {
        return updatable;
    }

    public void setUpdatable(boolean updatable) {
        this.updatable = updatable;
    }

    private void fireAddressMessagesEnabledChanged(Boolean isAddressMessagesEnabled) {
        for (NodeListener l : listeners) {
            l.addressMessagesEnabledChanged(isAddressMessagesEnabled);
        }
    }

    private void fireDccStartEnabledChanged(Boolean isDccStartEnabled) {
        for (NodeListener l : listeners) {
            l.dccStartEnabledChanged(isDccStartEnabled);
        }
    }

    private void fireFeedbackMessagesEnabledChanged(Boolean isFeedbackMessagesEnabled) {
        for (NodeListener l : listeners) {
            l.feedbackMessagesEnabledChanged(isFeedbackMessagesEnabled);
        }
    }

    private void fireIdentifyStateChanged(IdentifyState identifyState) {
        for (NodeListener l : listeners) {
            l.identifyStateChanged(identifyState);
        }
    }

    private void fireKeyMessagesEnabledChanged(Boolean isKeyMessagesEnabled) {
        for (NodeListener l : listeners) {
            l.keyMessagesEnabledChanged(isKeyMessagesEnabled);
        }
    }

    private void fireLabelChanged(String label) {
        for (NodeListener l : listeners) {
            l.labelChanged(label);
        }
    }

    public boolean equals(Object other) {
        LOGGER.debug("equals, other: {}", other);
        if (other != null) {
            return ((Node) other).hashCode() == hashCode();
        }
        return false;
    }

    public int hashCode() {
        if (node != null) {
            return Long.valueOf(getNode().getUniqueId()).hashCode();
        }
        return super.hashCode();
    }

    public String toString() {
        String result = null;

        if (label != null && label.length() > 0) {
            result = label;
        }
        else {
            result = node.getUniqueIdAsString();
        }
        return result;
    }
}
