package org.bidib.wizard.mvc.main.model;

import java.util.Collection;
import java.util.LinkedList;

import javax.swing.table.DefaultTableModel;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.listener.ServoPortListener;

public class ServoPortTableModel extends DefaultTableModel {
    private static final long serialVersionUID = 8451182315810784415L;

    private final Collection<ServoPortListener> servoPortListeners = new LinkedList<ServoPortListener>();

    private static final String[] COLUMN_NAMES =
        { Resources.getString(ServoPortTableModel.class, "label"),
            Resources.getString(ServoPortTableModel.class, "speed"),
            Resources.getString(ServoPortTableModel.class, "trimDown"),
            Resources.getString(ServoPortTableModel.class, "trimUp"),
            Resources.getString(ServoPortTableModel.class, "value"), null };

    public ServoPortTableModel() {
        setColumnIdentifiers(COLUMN_NAMES);
    }

    public void addServoPortListener(ServoPortListener l) {
        servoPortListeners.add(l);
    }

    public void addRow(ServoPort servoPort) {
        if (servoPort != null) {
            Object[] rowData = new Object[COLUMN_NAMES.length];

            rowData[0] = servoPort.toString();
            rowData[1] = servoPort.getSpeed();
            rowData[2] = servoPort.getTrimDown();
            rowData[3] = servoPort.getTrimUp();
            rowData[4] = 0;
            rowData[5] = servoPort;
            addRow(rowData);
        }
    }

    private void fireLabelChanged(ServoPort servoPort, String label) {
        for (ServoPortListener l : servoPortListeners) {
            l.labelChanged(servoPort, label);
        }
    }

    private void fireValuesChanged(ServoPort servoPort) {
        for (ServoPortListener l : servoPortListeners) {
            l.valuesChanged(servoPort);
        }
    }

    @Override
    public Class<?> getColumnClass(int column) {
        switch (column) {
            case 0:
                return String.class;
            default:
                return Integer.class;
        }
    }

    @Override
    public Object getValueAt(int row, int column) {
        Object result = super.getValueAt(row, column);

        if (result != null && column == 4) {
            ServoPort port = (ServoPort) getValueAt(row, 5);

            if (port != null) {
                result = port.getRelativeValue(Integer.parseInt(result.toString()));
            }
        }
        return result;
    }

    @Override
    public void setValueAt(Object value, int row, int column) {
        final ServoPort port = (ServoPort) getValueAt(row, 5);

        switch (column) {
            case 0:
                port.setLabel((String) value);
                super.setValueAt(port.toString(), row, column);
                fireLabelChanged(port, port.getLabel());
                break;
            case 1:
                try {
                    port.setSpeed(Integer.parseInt(value.toString()));
                    super.setValueAt(value, row, column);
                    fireValuesChanged(port);
                }
                catch (NumberFormatException e) {
                }
                break;
            case 2:
                if (port.getTrimDown() != (Integer) value) {
                    port.setTrimDown((Integer) value);
                    super.setValueAt(value, row, column);
                    fireValuesChanged(port);
                }
                break;
            case 3:
                if (port.getTrimUp() != (Integer) value) {
                    port.setTrimUp((Integer) value);
                    super.setValueAt(value, row, column);
                    fireValuesChanged(port);
                }
                break;
            case 4:
                int intValue = port.getAbsoluteValue(Integer.parseInt(value.toString()));

                if (port.getValue() != intValue) {
                    port.setValue(intValue);
                    super.setValueAt(intValue, row, column);
                }
                break;
        }
    }
}
