package org.bidib.wizard.mvc.main.view.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.Accessory;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.listener.AccessoryListListener;
import org.bidib.wizard.mvc.main.model.listener.AccessoryListener;
import org.bidib.wizard.mvc.main.view.menu.listener.AccessoryTableMenuListener;

public class AccessoryTableMenu extends MyPopupMenu implements AccessoryListener {
    private static final long serialVersionUID = -8916729225855312933L;

    public static final KeyStroke KEYSTROKE_DELETE = KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0);

    private final Collection<AccessoryTableMenuListener> menuListeners = new LinkedList<AccessoryTableMenuListener>();

    private final JMenuItem insertEmptyBefore = new JMenuItem(Resources.getString(getClass(), "insertEmptyBefore"));

    private final JMenuItem insertEmptyAfter = new JMenuItem(Resources.getString(getClass(), "insertEmptyAfter"));

    private final JMenuItem delete = new JMenuItem(Resources.getString(getClass(), "delete"));

    private final JMenuItem selectAll = new JMenuItem(Resources.getString(getClass(), "selectAll"));

    private Accessory accessory = null;

    public AccessoryTableMenu(final MainModel model) {
        insertEmptyBefore.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireInsertEmptyBefore();
            }
        });
        add(insertEmptyBefore);

        insertEmptyAfter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireInsertEmptyAfter();
            }
        });
        add(insertEmptyAfter);

        delete.setAccelerator(KEYSTROKE_DELETE);
        delete.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireDelete();
            }
        });
        add(delete);

        addSeparator();

        selectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        selectAll.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireSelectAll();
            }
        });
        selectAll.setEnabled(false);
        add(selectAll);

        model.addAccessoryListListener(new AccessoryListListener() {
            @Override
            public void accessoryChanged() {
                Accessory accessory = getAccessory();

                if (accessory != null) {
                    accessory.removeAccessoryListener(AccessoryTableMenu.this);
                }

                accessory = model.getAccessory();

                enableMenuItems(accessory);
                if (accessory != null) {
                    accessory.addAccessoryListener(AccessoryTableMenu.this);
                }
                setAccessory(accessory);
            }

            @Override
            public void listChanged() {
            }
        });
    }

    public void addMenuListener(AccessoryTableMenuListener l) {
        menuListeners.add(l);
    }

    private void enableMenuItems(Accessory accessory) {
        if (accessory != null) {
            boolean accessoryEmpty = accessory.getMacroCount() == 0;
            boolean accessoryFull = accessory.getMacroCount() >= accessory.getMacroSize();

            insertEmptyBefore.setEnabled(!accessoryFull);
            insertEmptyAfter.setEnabled(!accessoryFull);
            delete.setEnabled(!accessoryEmpty);
            selectAll.setEnabled(!accessoryEmpty);
        }
        else {
            insertEmptyBefore.setEnabled(true);
            insertEmptyAfter.setEnabled(true);
            delete.setEnabled(false);
            selectAll.setEnabled(false);
        }
    }

    public void fireDelete() {
        for (AccessoryTableMenuListener l : menuListeners) {
            l.delete();
        }
    }

    private void fireInsertEmptyAfter() {
        for (AccessoryTableMenuListener l : menuListeners) {
            l.insertEmptyAfter();
        }
    }

    private void fireInsertEmptyBefore() {
        for (AccessoryTableMenuListener l : menuListeners) {
            l.insertEmptyBefore();
        }
    }

    private void fireSelectAll() {
        for (AccessoryTableMenuListener l : menuListeners) {
            l.selectAll();
        }
    }

    public Accessory getAccessory() {
        return accessory;
    }

    public void setAccessory(Accessory accessory) {
        this.accessory = accessory;
    }

    @Override
    public void labelChanged(String label) {
    }

    @Override
    public void macrosChanged() {
        enableMenuItems(accessory);
    }
}
