package org.bidib.wizard.mvc.main.view.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.Box;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import org.bidib.jbidibc.enumeration.IdentifyState;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.Node;
import org.bidib.wizard.mvc.main.model.listener.NodeListListener;
import org.bidib.wizard.mvc.main.model.listener.NodeListener;
import org.bidib.wizard.mvc.main.view.menu.listener.MainMenuListener;

public class MainMenuBar extends JMenuBar implements NodeListener {
    private static final long serialVersionUID = 1864162731138661563L;

    private final Collection<MainMenuListener> menuListeners = new LinkedList<MainMenuListener>();

    private final NodeListMenu nodeListMenu;

    private Node node = null;

    public MainMenuBar(final MainModel model) {
        nodeListMenu = new NodeListMenu(model);

        JMenu fileMenu = new JMenu(Resources.getString(getClass(), "file"));
        JMenuItem exitItem = new JMenuItem(Resources.getString(getClass(), "quit"));

        exitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
        exitItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireExit();
            }
        });
        fileMenu.add(exitItem);
        add(fileMenu);

        JMenu editMenu = new JMenu(Resources.getString(getClass(), "edit"));
        JMenuItem preferencesItem = new JMenuItem(Resources.getString(getClass(), "preferences") + " ...");

        preferencesItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                firePreferences();
            }
        });
        editMenu.add(preferencesItem);
        add(editMenu);

        final JMenu nodeMenu = nodeListMenu.getJMenu();

        nodeMenu.addMenuListener(new MenuListener() {
            @Override
            public void menuCanceled(MenuEvent e) {
            }

            @Override
            public void menuDeselected(MenuEvent e) {
            }

            @Override
            public void menuSelected(MenuEvent e) {
                boolean hasData = nodeListMenu.hasData();

                nodeListMenu.setImportEnabled(hasData);
                nodeListMenu.setExportEnabled(hasData);
                nodeListMenu.setSaveEnabled(hasData);
            }
        });
        nodeMenu.setEnabled(false);
        nodeMenu.setText(Resources.getString(getClass(), "node"));
        add(nodeMenu);

        model.addNodeListListener(new NodeListListener() {
            @Override
            public void listChanged() {
            }

            @Override
            public void nodeChanged() {
                Node node = getNode();

                if (node != null) {
                    node.removeNodeListener(MainMenuBar.this);
                }

                Node newNode = model.getNode();

                nodeMenu.setEnabled(newNode != null);
                if (newNode != null) {
                    nodeListMenu.setAddressMessagesEnabled(newNode.isAddressMessagesEnabled());
                    nodeListMenu.setDccStartEnabled(newNode.isDccStartEnabled());
                    nodeListMenu.setExternalStartEnabled(newNode.isExternalStartEnabled());
                    nodeListMenu.setFeedbackMessagesEnabled(newNode.isFeedbackMessagesEnabled());
                    nodeListMenu.setFirmwareUpdateEnabled(newNode.isUpdatable());
                    nodeListMenu.setIdentifyStateSelected(newNode.getIdentifyState());
                    nodeListMenu.setKeyMessagesEnabled(newNode.isKeyMessagesEnabled());
                    nodeListMenu.setLocoEnabled(newNode.isCommandStation());
                    newNode.addNodeListener(MainMenuBar.this);
                }
                setNode(newNode);
            }
        });

        add(Box.createHorizontalGlue());

        JMenu helpMenu = new JMenu(Resources.getString(getClass(), "help"));
        JMenuItem aboutItem = new JMenuItem(Resources.getString(getClass(), "about") + " ...");

        aboutItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireAbout();
            }
        });
        helpMenu.add(aboutItem);
        add(helpMenu);
    }

    @Override
    public void addressMessagesEnabledChanged(Boolean isAddressMessagesEnabled) {
        nodeListMenu.setAddressMessagesEnabled(isAddressMessagesEnabled);
    }

    public void addMenuListener(MainMenuListener l) {
        menuListeners.add(l);
    }

    @Override
    public void dccStartEnabledChanged(Boolean isDccStartEnabled) {
        nodeListMenu.setDccStartEnabled(isDccStartEnabled);
    }

    @Override
    public void externalStartEnabledChanged(Boolean isExternalStartEnabled) {
        nodeListMenu.setExternalStartEnabled(isExternalStartEnabled);
    }

    @Override
    public void feedbackMessagesEnabledChanged(Boolean isFeedbackMessagesEnabled) {
        nodeListMenu.setFeedbackMessagesEnabled(isFeedbackMessagesEnabled);
    }

    private void fireAbout() {
        for (MainMenuListener l : menuListeners) {
            l.about();
        }
    }

    private void fireExit() {
        for (MainMenuListener l : menuListeners) {
            l.exit();
        }
    }

    private void firePreferences() {
        for (MainMenuListener l : menuListeners) {
            l.preferences();
        }
    }

    private Node getNode() {
        return node;
    }

    @Override
    public void identifyStateChanged(IdentifyState identifyState) {
        nodeListMenu.setIdentifyStateSelected(identifyState);
    }

    @Override
    public void keyMessagesEnabledChanged(Boolean isKeyMessagesEnabled) {
        nodeListMenu.setKeyMessagesEnabled(isKeyMessagesEnabled);
    }

    @Override
    public void labelChanged(String label) {
    }

    private void setNode(Node node) {
        this.node = node;
    }
}
