package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.common.view.panel.DisabledPanel;
import org.bidib.wizard.mvc.main.model.Accessory;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.view.menu.AccessoryListMenu;
import org.bidib.wizard.mvc.main.view.menu.listener.AccessoryListMenuListener;
import org.bidib.wizard.mvc.main.view.panel.listener.AccessoryListListener;
import org.bidib.wizard.mvc.main.view.panel.listener.AccessoryTableListener;
import org.bidib.wizard.mvc.main.view.panel.listener.LabelChangedListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AccessoryListPanel extends LabelListPanel implements AccessoryListMenuListener {
    private static final Logger LOGGER = LoggerFactory.getLogger(AccessoryListPanel.class);

    private static final long serialVersionUID = -511163701919493545L;

    private static final String EMPTY_LABEL = " ";

    private final Collection<AccessoryListListener> accessoryListListeners = new LinkedList<AccessoryListListener>();

    private final MainModel model;

    private final AccessoryPanel accessoryPanel;

    private final AccessoryListMenu accessoryListMenu = new AccessoryListMenu();

    private final JPanel borderPanel = new JPanel(new GridBagLayout());

    private final TitledBorder border =
        BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), EMPTY_LABEL);

    private final DisabledPanel disabledBorderPanel;

    public AccessoryListPanel(final MainModel model) {
        this.model = model;
        setLayout(new GridBagLayout());
        getLabelList().setListData(model.getAccessories().toArray());
        getLabelList().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        accessoryListMenu.addMenuListener(this);

        borderPanel.setBorder(border);

        accessoryPanel = new AccessoryPanel(model);

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.fill = GridBagConstraints.BOTH;
        c.gridheight = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.9;
        c.weighty = 1;
        borderPanel.add(accessoryPanel, c);

        JPanel panel = new JPanel(new GridBagLayout());

        c.anchor = GridBagConstraints.LINE_START;
        c.gridx = 1;
        panel.add(borderPanel, c);
        disabledBorderPanel = new DisabledPanel(borderPanel);
        panel.add(disabledBorderPanel, c);

        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, new JScrollPane(getLabelList()), panel);

        splitPane.setContinuousLayout(true);
        splitPane.setOneTouchExpandable(true);
        splitPane.setResizeWeight(0.2);

        add(splitPane, c);
        setEnabled(false);

        addLabelChangedListener(new LabelChangedListener() {
            @Override
            public void labelChanged(Object object, String label) {
                setLabel(object.toString());
            }
        });
        addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    final Accessory accessory = (Accessory) ((JList<?>) e.getSource()).getSelectedValue();

                    setLabel(accessory != null ? accessory.toString() : null);
                    setEnabled(accessory != null);
                }
            }
        });
        model.addAccessoryListListener(new org.bidib.wizard.mvc.main.model.listener.AccessoryListListener() {
            @Override
            public void listChanged() {
                getLabelList().setListData(model.getAccessories().toArray());
                setEnabled(false);
            }

            @Override
            public void accessoryChanged() {
                Accessory accessory = model.getAccessory();

                setLabel(accessory != null ? accessory.toString() : null);
                setEnabled(accessory != null);
            }
        });
        getLabelList().addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                handleMouseEvent(e, accessoryListMenu);
            }

            public void mouseReleased(MouseEvent e) {
                handleMouseEvent(e, accessoryListMenu);
            }
        });
        addButtonPanel(c);
    }

    public void addAccessoryListListener(AccessoryListListener l) {
        addLabelChangedListener(l);
        accessoryListListeners.add(l);
    }

    public void addAccessoryTableListener(AccessoryTableListener l) {
        accessoryPanel.addTableListener(l);
    }

    private void addButtonPanel(GridBagConstraints c) {
        JPanel buttonPanel = new JPanel(new GridBagLayout());

        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.NONE;
        c.gridwidth = 1;
        c.weightx = 0.33;
        c.weighty = 0;

        JButton loadButton = new JButton(Resources.getString(getClass(), "reload"));

        loadButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireReloadAccessory();
            }
        });
        buttonPanel.add(loadButton, c);

        c.anchor = GridBagConstraints.LINE_END;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx++;

        JButton saveButton = new JButton(Resources.getString(getClass(), "save"));

        saveButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireSaveAccessory();
            }
        });
        buttonPanel.add(saveButton, c);

        c.anchor = GridBagConstraints.LAST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy++;
        borderPanel.add(buttonPanel, c);
    }

    @Override
    public void exportAccessory() {
        for (AccessoryListListener l : accessoryListListeners) {
            l.exportAccessory(model.getAccessory());
        }
    }

    private void fireReloadAccessory() {
        for (AccessoryListListener l : accessoryListListeners) {
            l.reloadAccessory(model.getAccessory());
        }
    }

    private void fireSaveAccessory() {
        for (AccessoryListListener l : accessoryListListeners) {
            l.saveAccessory(model.getAccessory());
        }
    }

    public String getName() {
        return Resources.getString(getClass(), "name");
    }

    @Override
    public void importAccessory() {
        for (AccessoryListListener l : accessoryListListeners) {
            l.importAccessory(model.getAccessory());
        }
    }

    public void setEnabled(boolean enabled) {
        disabledBorderPanel.setEnabled(enabled);
    }

    private void setLabel(String label) {
        // TODO check if called from non-AWT-thread
        LOGGER.debug("Set the label: {}", label);

        if (label != null && !label.equals(EMPTY_LABEL)) {
            border.setTitle(label + ":");
        }
        else {
            border.setTitle("");
        }
        borderPanel.repaint();
    }
}
