package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.bidib.wizard.comm.AnalogPortStatus;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.AccessoryTableModel;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.listener.AccessoryPortListener;
import org.bidib.wizard.mvc.main.view.menu.AccessoryTableMenu;
import org.bidib.wizard.mvc.main.view.menu.listener.AccessoryTableMenuListener;
import org.bidib.wizard.mvc.main.view.panel.listener.AccessoryTableListener;
import org.bidib.wizard.mvc.main.view.table.ComboBoxEditor;
import org.bidib.wizard.mvc.main.view.table.ComboBoxRenderer;
import org.bidib.wizard.mvc.main.view.table.ComboBoxWithButtonEditor;
import org.bidib.wizard.mvc.main.view.table.ComboBoxWithButtonRenderer;
import org.bidib.wizard.mvc.main.view.table.MyTable;

public class AccessoryPanel extends JPanel {
    private static final long serialVersionUID = 8431575212722895211L;

    private final Collection<AccessoryTableListener> tableListeners = new LinkedList<AccessoryTableListener>();

    private final MyTable table;

    private final AccessoryTableMenu accessoryTableMenu;

    public AccessoryPanel(final MainModel model) {
        setLayout(new GridBagLayout());
        accessoryTableMenu = new AccessoryTableMenu(model);
        accessoryTableMenu.addMenuListener(new AccessoryTableMenuListener() {
            @Override
            public void delete() {
                fireDelete(table.getSelectedRows());
            }

            @Override
            public void insertEmptyAfter() {
                fireInsertEmptyAfter(table.getSelectedRow());
            }

            @Override
            public void insertEmptyBefore() {
                fireInsertEmptyBefore(table.getSelectedRow());
            }

            @Override
            public void selectAll() {
                int rowCount = table.getRowCount();

                if (rowCount > 0) {
                    table.setRowSelectionInterval(0, rowCount - 1);
                }
            }
        });

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);

        AccessoryTableModel tableModel = new AccessoryTableModel(model);

        table = new MyTable(
            tableModel, Resources.getString(getClass(), "emptyTable")) {
            private static final long serialVersionUID = -2524411283898766237L;

            public TableCellEditor getCellEditor(int row, int column) {
                TableCellEditor result = super.getCellEditor(row, column);

                if (column == 1) {
                    result = new ComboBoxEditor(model.getMacros().toArray());
                }
                else if (column == 2) {
                    result = new ComboBoxWithButtonEditor(table.getActions(AnalogPortStatus.START), ">");
                }
                return result;
            }

            public TableCellRenderer getCellRenderer(int row, int column) {
                TableCellRenderer result = super.getCellRenderer(row, column);

                if (column == 1) {
                    result = new ComboBoxRenderer(model.getMacros().toArray());
                }
                else if (column == 2) {
                    result = new ComboBoxWithButtonRenderer(table.getActions(AnalogPortStatus.START), ">");
                }
                return result;
            }
        };
        table.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                handleMouseEvent(e, accessoryTableMenu);
            }

            public void mouseReleased(MouseEvent e) {
                handleMouseEvent(e, accessoryTableMenu);
            }
        });
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(AccessoryTableMenu.KEYSTROKE_DELETE,
            "delete");
        table.getActionMap().put("delete", new AbstractAction() {
            private static final long serialVersionUID = -1355565200937532853L;

            @Override
            public void actionPerformed(ActionEvent e) {
                accessoryTableMenu.fireDelete();
            }
        });
        tableModel.addPortListener(new AccessoryPortListener() {
            @Override
            public void testButtonPressed(int aspect) {
                fireTestButtonPressed(aspect);
            }
        });
        add(new JScrollPane(table), c);
    }

    public void addTableListener(AccessoryTableListener l) {
        tableListeners.add(l);
    }

    private void fireDelete(int[] rows) {
        for (AccessoryTableListener l : tableListeners) {
            l.delete(rows);
        }
    }

    private void fireInsertEmptyAfter(int row) {
        for (AccessoryTableListener l : tableListeners) {
            l.insertEmptyAfter(row);
        }
    }

    private void fireInsertEmptyBefore(int row) {
        for (AccessoryTableListener l : tableListeners) {
            l.insertEmptyBefore(row);
        }
    }

    private void fireTestButtonPressed(int row) {
        for (AccessoryTableListener l : tableListeners) {
            l.testButtonPressed(row);
        }
    }

    private int getRow(Point point) {
        return table.rowAtPoint(point);
    }

    private void handleMouseEvent(MouseEvent e, JPopupMenu popupMenu) {
        if (e.isPopupTrigger()) {
            int row = getRow(e.getPoint());

            if (table.getSelectedRowCount() == 0 && table.getRowCount() > 0 && row >= 0 && row < table.getRowCount()) {
                table.setRowSelectionInterval(row, row);
            }
            table.grabFocus();
            popupMenu.show(e.getComponent(), e.getX(), e.getY());
        }
    }
}
