package org.bidib.wizard.mvc.main.view.panel;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.event.ListSelectionListener;

import org.bidib.wizard.dialog.LabelDialog;
import org.bidib.wizard.mvc.main.model.LabelAware;
import org.bidib.wizard.mvc.main.view.component.LabelList;
import org.bidib.wizard.mvc.main.view.menu.listener.LabelListMenuListener;
import org.bidib.wizard.mvc.main.view.panel.listener.LabelChangedListener;

public abstract class LabelListPanel extends JPanel implements LabelListMenuListener {
    private static final long serialVersionUID = -1169499687558371766L;

    private final Collection<LabelChangedListener> labelChangedListeners = new LinkedList<LabelChangedListener>();

    private final LabelList labelList = new LabelList();

    protected LabelList getLabelList() {
        return labelList;
    }

    public void addLabelChangedListener(LabelChangedListener l) {
        labelChangedListeners.add(l);
    }

    public void addListSelectionListener(ListSelectionListener l) {
        labelList.addListSelectionListener(l);
    }

    @Override
    public void editLabel() {
        int selectedIndex = labelList.getSelectedIndex();

        if (selectedIndex >= 0) {
            final Object object = labelList.getModel().getElementAt(selectedIndex);
            final Point itemPosition = selectedIndexToLocation();

            new LabelDialog(
                object.toString(), itemPosition.x, itemPosition.y) {
                @Override
                public void labelChanged(String value) {
                    if (object instanceof LabelAware) {
                        ((LabelAware) object).setLabel(value);
                    }
                    fireLabelChanged(object, value);
                }
            };
        }
    }

    protected void fireLabelChanged(Object object, String label) {
        for (LabelChangedListener l : labelChangedListeners) {
            l.labelChanged(object, label);
        }
    }

    protected void handleMouseEvent(MouseEvent e, JPopupMenu popupMenu) {
        if (e.isPopupTrigger() && labelList.getModel().getSize() > 0) {
            labelList.setSelectedIndex(labelList.getIndex(e.getPoint()));
            labelList.grabFocus();
            popupMenu.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    protected Point selectedIndexToLocation() {
        Point result = new Point();
        int selectedIndex = labelList.getSelectedIndex();

        if (selectedIndex >= 0) {
            final Point listPosition = labelList.getLocationOnScreen();
            final Point itemPosition = labelList.indexToLocation(selectedIndex);
            result.setLocation(listPosition.x + itemPosition.x, listPosition.y + itemPosition.y);
        }
        return result;
    }
}
