package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.AbstractAction;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.bidib.wizard.comm.LightPortStatus;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.LightPort;
import org.bidib.wizard.mvc.main.model.LightPortTableModel;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.Port;
import org.bidib.wizard.mvc.main.model.listener.LightPortListener;
import org.bidib.wizard.mvc.main.model.listener.PortListListener;
import org.bidib.wizard.mvc.main.view.table.ComboBoxWithButtonEditor;
import org.bidib.wizard.mvc.main.view.table.ComboBoxWithButtonRenderer;
import org.bidib.wizard.mvc.main.view.table.PortTable;
import org.bidib.wizard.mvc.main.view.table.SliderEditor;
import org.bidib.wizard.mvc.main.view.table.TwoDimensionalArray;
import org.bidib.wizard.mvc.main.view.table.listener.ButtonListener;

public class LightPortListPanel extends SimplePortListPanel implements ButtonListener, PortListListener {
    private static final long serialVersionUID = 5506124079986633240L;

    private final Collection<LightPortListener> lightPortListeners = new LinkedList<LightPortListener>();

    private final MainModel model;

    private final LightPortTableModel tableModel;

    public LightPortListPanel(MainModel model) {
        super(null, null, null);
        this.model = model;
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);

        tableModel = new LightPortTableModel();
        table = new PortTable(
            tableModel, Resources.getString(getClass(), "emptyTable")) {
            private static final long serialVersionUID = 7243578986105476486L;

            private final TwoDimensionalArray<SliderEditor> sliderEditors = new TwoDimensionalArray<SliderEditor>();

            public TableCellEditor getCellEditor(int row, int column) {
                TableCellEditor result = null;

                if (column >= 1 && column <= 4) {
                    result = sliderEditors.get(0, column);
                }
                else if (column == 5) {
                    ComboBoxWithButtonEditor editor = new ComboBoxWithButtonEditor(getActions(LightPortStatus.ON), ">");

                    editor.addButtonListener(LightPortListPanel.this);
                    result = editor;
                }
                else {
                    result = super.getCellEditor(row, column);
                }
                return result;
            }

            public TableCellRenderer getCellRenderer(int row, int column) {
                TableCellRenderer result = null;

                if (column >= 1 && column <= 4) {
                    SliderEditor sliderEditor = new SliderEditor();

                    sliderEditors.set(sliderEditor, 0, column);
                    result = sliderEditor;
                }
                else if (column == 5) {
                    result = new ComboBoxWithButtonRenderer(getActions(LightPortStatus.ON), ">");
                }
                else {
                    result = super.getCellRenderer(row, column);
                }
                return result;
            }
        };
        table.removeColumn(table.getColumnModel().getColumn(table.getColumnCount() - 1));
        table.setRowSelectionAllowed(false);

        // let the left and right key change the slider value
        table.getInputMap(JTable.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0),
            "left");
        table.getActionMap().put("left", new AbstractAction() {
            private static final long serialVersionUID = -943043159825570302L;

            @Override
            public void actionPerformed(ActionEvent e) {
                JTable table = (JTable) e.getSource();
                TableCellEditor editor = table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn());

                if (editor instanceof SliderEditor) {
                    table.editCellAt(table.getSelectedRow(), table.getSelectedColumn());
                    ((SliderEditor) editor).setDown();
                }
            }
        });
        table.getInputMap(JTable.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0),
            "right");
        table.getActionMap().put("right", new AbstractAction() {
            private static final long serialVersionUID = -943043159825570302L;

            @Override
            public void actionPerformed(ActionEvent e) {
                JTable table = (JTable) e.getSource();
                TableCellEditor editor = table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn());

                if (editor instanceof SliderEditor) {
                    table.editCellAt(table.getSelectedRow(), table.getSelectedColumn());
                    ((SliderEditor) editor).setUp();
                }
            }
        });

        add(new JScrollPane(table), c);

        model.addLightPortListListener(this);
    }

    public void addLightPortListener(LightPortListener l) {
        tableModel.addLightPortListener(l);
        lightPortListeners.add(l);
    }

    @Override
    public void buttonPressed(int row, int column) {
        for (LightPortListener l : lightPortListeners) {
            l.testButtonPressed((Port) table.getModel().getValueAt(row, table.getModel().getColumnCount() - 1));
        }
    }

    @Override
    public void listChanged() {
        tableModel.setRowCount(0);
        for (LightPort port : model.getLightPorts()) {
            tableModel.addRow(port);
        }
        table.packColumn(table.getColumnCount() - 1, 2);
    }
}
