package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.AnalogPort;
import org.bidib.wizard.mvc.main.model.Flag;
import org.bidib.wizard.mvc.main.model.InputPort;
import org.bidib.wizard.mvc.main.model.LightPort;
import org.bidib.wizard.mvc.main.model.Macro;
import org.bidib.wizard.mvc.main.model.MacroTableModel;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.MotorPort;
import org.bidib.wizard.mvc.main.model.ServoPort;
import org.bidib.wizard.mvc.main.model.SoundPort;
import org.bidib.wizard.mvc.main.model.SwitchPort;
import org.bidib.wizard.mvc.main.model.function.AnalogPortAction;
import org.bidib.wizard.mvc.main.model.function.CriticalFunction;
import org.bidib.wizard.mvc.main.model.function.DelayFunction;
import org.bidib.wizard.mvc.main.model.function.Delayable;
import org.bidib.wizard.mvc.main.model.function.FlagFunction;
import org.bidib.wizard.mvc.main.model.function.Function;
import org.bidib.wizard.mvc.main.model.function.InputFunction;
import org.bidib.wizard.mvc.main.model.function.LightPortAction;
import org.bidib.wizard.mvc.main.model.function.MacroFunction;
import org.bidib.wizard.mvc.main.model.function.MotorPortAction;
import org.bidib.wizard.mvc.main.model.function.RandomDelayFunction;
import org.bidib.wizard.mvc.main.model.function.ServoPortAction;
import org.bidib.wizard.mvc.main.model.function.SoundPortAction;
import org.bidib.wizard.mvc.main.model.function.SwitchPortAction;
import org.bidib.wizard.mvc.main.model.listener.MacroListListener;
import org.bidib.wizard.mvc.main.view.menu.AccessoryTableMenu;
import org.bidib.wizard.mvc.main.view.menu.MacroTableMenu;
import org.bidib.wizard.mvc.main.view.menu.listener.MacroTableMenuListener;
import org.bidib.wizard.mvc.main.view.panel.listener.MacroTableListener;
import org.bidib.wizard.mvc.main.view.table.ComboBoxEditor;
import org.bidib.wizard.mvc.main.view.table.ComboBoxRenderer;
import org.bidib.wizard.mvc.main.view.table.MyTable;
import org.bidib.wizard.mvc.main.view.table.NumberWithLabelEditor;
import org.bidib.wizard.mvc.main.view.table.NumberWithLabelRenderer;

public class MacroContentPanel extends JPanel {
    private static final long serialVersionUID = -1488235261101761514L;

    private final Collection<MacroTableListener> tableListeners = new LinkedList<MacroTableListener>();

    private final MainModel model;

    private final MacroTableModel tableModel;

    private final MyTable table;

    private final MacroTableMenu macroTableMenu;

    public MacroContentPanel(final MainModel model) {
        this.macroTableMenu = new MacroTableMenu(model);
        this.model = model;
        setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(getClass(),
            "title")
            + ":"));
        setLayout(new GridBagLayout());

        macroTableMenu.addMenuListener(new MacroTableMenuListener() {
            @Override
            public void copy() {
                fireCopy(table.getSelectedRows());
            }

            @Override
            public void cut() {
                fireCut(table.getSelectedRows());
            }

            @Override
            public void delete() {
                fireDelete(table.getSelectedRows());
            }

            @Override
            public void insertEmptyAfter() {
                fireInsertEmptyAfter(table.getSelectedRow());
            }

            @Override
            public void insertEmptyBefore() {
                fireInsertEmptyBefore(table.getSelectedRow());
            }

            @Override
            public void pasteAfter() {
                firePasteAfter(table.getSelectedRow());
            }

            @Override
            public void pasteBefore() {
                firePasteBefore(table.getSelectedRow());
            }

            @Override
            public void selectAll() {
                int rowCount = table.getRowCount();

                if (rowCount > 0) {
                    table.setRowSelectionInterval(0, rowCount - 1);
                }
            }
        });

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);

        tableModel = new MacroTableModel(model);
        table = new MyTable(
            tableModel, Resources.getString(getClass(), "emptyTable")) {
            private static final long serialVersionUID = -1726592086857419535L;

            public TableCellEditor getCellEditor(int row, int column) {
                TableCellEditor result = null;
                Function function = model.getMacro().getFunction(row);

                switch (column) {
                    case 1:
                        if (function instanceof Delayable) {
                            result =
                                new NumberWithLabelEditor(Resources.getString(MacroContentPanel.class, "delay") + ":",
                                    "Ticks");
                        }
                        else if (function instanceof RandomDelayFunction) {
                            result =
                                new NumberWithLabelEditor(
                                    Resources.getString(MacroContentPanel.class, "maximum") + ":", "Ticks");
                        }
                        break;
                    case 2:
                        result = new ComboBoxEditor(getFunctions());
                        break;
                    case 3:
                        if (function != null) {
                            Object[] actions = getActions(function.getAction());

                            if (actions.length > 1) {
                                result = new ComboBoxEditor(getActions(function.getAction()));
                            }
                        }
                        break;
                    case 4:
                        if (function instanceof AnalogPortAction) {
                            result = new ComboBoxEditor(model.getAnalogPorts().toArray(new AnalogPort[0]));
                        }
                        else if (function instanceof FlagFunction) {
                            result = new ComboBoxEditor(model.getFlags().toArray(new Flag[0]));
                        }
                        else if (function instanceof InputFunction) {
                            result = new ComboBoxEditor(model.getInputPorts().toArray(new InputPort[0]));
                        }
                        else if (function instanceof LightPortAction) {
                            result = new ComboBoxEditor(model.getLightPorts().toArray(new LightPort[0]));
                        }
                        else if (function instanceof MacroFunction) {
                            result = new ComboBoxEditor(model.getMacros().toArray(new Macro[0]));
                        }
                        else if (function instanceof MotorPortAction) {
                            result = new ComboBoxEditor(model.getMotorPorts().toArray(new MotorPort[0]));
                        }
                        else if (function instanceof ServoPortAction) {
                            result = new ComboBoxEditor(model.getServoPorts().toArray(new ServoPort[0]));
                        }
                        else if (function instanceof SoundPortAction) {
                            result = new ComboBoxEditor(model.getSoundPorts().toArray(new SoundPort[0]));
                        }
                        else if (function instanceof SwitchPortAction) {
                            result = new ComboBoxEditor(model.getSwitchPorts().toArray(new SwitchPort[0]));
                        }
                        break;
                    case 5:
                        if (function instanceof ServoPortAction) {
                            result =
                                new NumberWithLabelEditor(Resources.getString(MacroContentPanel.class, "destination")
                                    + ":", "%");
                        }
                        break;
                }
                return result;
            }

            public TableCellRenderer getCellRenderer(int row, int column) {
                TableCellRenderer result = super.getCellRenderer(row, column);

                if (model.getMacro() != null && row >= 0 && row < model.getMacro().getFunctions().size()) {
                    Function function = model.getMacro().getFunction(row);

                    switch (column) {
                        case 1:
                            if (function instanceof Delayable) {
                                result =
                                    new NumberWithLabelRenderer(Resources.getString(MacroContentPanel.class, "delay")
                                        + ":", "Ticks");
                            }
                            else if (function instanceof RandomDelayFunction) {
                                result =
                                    new NumberWithLabelRenderer(Resources.getString(MacroContentPanel.class, "maximum")
                                        + ":", "Ticks");
                            }
                            break;
                        case 2:
                            result = new ComboBoxRenderer(getFunctions());
                            break;
                        case 3:
                            if (function != null) {
                                Object[] actions = getActions(function.getAction());

                                if (actions.length > 1) {
                                    result = new ComboBoxRenderer(actions);
                                }
                            }
                            break;
                        case 4:
                            if (function instanceof AnalogPortAction) {
                                result = new ComboBoxRenderer(model.getAnalogPorts().toArray(new AnalogPort[0]));
                            }
                            else if (function instanceof FlagFunction) {
                                result = new ComboBoxRenderer(model.getFlags().toArray(new Flag[0]));
                            }
                            else if (function instanceof InputFunction) {
                                result = new ComboBoxRenderer(model.getInputPorts().toArray(new InputPort[0]));
                            }
                            else if (function instanceof LightPortAction) {
                                result = new ComboBoxRenderer(model.getLightPorts().toArray(new LightPort[0]));
                            }
                            else if (function instanceof MacroFunction) {
                                result = new ComboBoxRenderer(model.getMacros().toArray(new Macro[0]));
                            }
                            else if (function instanceof MotorPortAction) {
                                result = new ComboBoxRenderer(model.getMotorPorts().toArray(new MotorPort[0]));
                            }
                            else if (function instanceof ServoPortAction) {
                                result = new ComboBoxRenderer(model.getServoPorts().toArray(new ServoPort[0]));
                            }
                            else if (function instanceof SoundPortAction) {
                                result = new ComboBoxRenderer(model.getSoundPorts().toArray(new SoundPort[0]));
                            }
                            else if (function instanceof SwitchPortAction) {
                                result = new ComboBoxRenderer(model.getSwitchPorts().toArray(new SwitchPort[0]));
                            }
                            break;
                        case 5:
                            if (function instanceof ServoPortAction) {
                                result =
                                    new NumberWithLabelRenderer(Resources.getString(MacroContentPanel.class,
                                        "destination")
                                        + ":", "%");
                            }
                            break;
                    }
                }
                return result;
            }
        };
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(MacroTableMenu.KEYSTROKE_CUT, "cut");
        table.getActionMap().put("cut", new AbstractAction() {
            private static final long serialVersionUID = -2494399401765228099L;

            @Override
            public void actionPerformed(ActionEvent e) {
                macroTableMenu.fireCut();
            }
        });
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(MacroTableMenu.KEYSTROKE_COPY, "copy");
        table.getActionMap().put("copy", new AbstractAction() {
            private static final long serialVersionUID = -2494399401765228099L;

            @Override
            public void actionPerformed(ActionEvent e) {
                macroTableMenu.fireCopy();
            }
        });
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(MacroTableMenu.KEYSTROKE_PASTE, "paste");
        table.getActionMap().put("paste", new AbstractAction() {
            private static final long serialVersionUID = 8369200342378372047L;

            @Override
            public void actionPerformed(ActionEvent e) {
                macroTableMenu.firePasteAfter();
            }
        });
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(AccessoryTableMenu.KEYSTROKE_DELETE,
            "delete");
        table.getActionMap().put("delete", new AbstractAction() {
            private static final long serialVersionUID = 8369200342378372047L;

            @Override
            public void actionPerformed(ActionEvent e) {
                macroTableMenu.fireDelete();
            }
        });
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        table.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                handleMouseEvent(e, macroTableMenu);
            }

            public void mouseReleased(MouseEvent e) {
                handleMouseEvent(e, macroTableMenu);
            }
        });
        table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        table.setTableHeader(null);
        add(new JScrollPane(table), c);

        model.addMacroListListener(new MacroListListener() {
            @Override
            public void listChanged() {
                tableModel.setRowCount(0);
            }

            @Override
            public void macroChanged() {
                Macro macro = model.getMacro();

                if (macro != null && macro.getFunctions() != null) {
                    tableModel.setMacro(macro);
                }
            }
        });
    }

    public void addTableListener(MacroTableListener l) {
        tableListeners.add(l);
    }

    private void fireCopy(int[] rows) {
        for (MacroTableListener l : tableListeners) {
            l.copy(getFunctions(rows));
        }
    }

    private void fireCut(int[] rows) {
        for (MacroTableListener l : tableListeners) {
            l.cut(rows, getFunctions(rows));
        }
    }

    private void fireDelete(int[] rows) {
        for (MacroTableListener l : tableListeners) {
            l.delete(rows);
        }
    }

    private void fireInsertEmptyAfter(int row) {
        for (MacroTableListener l : tableListeners) {
            l.insertEmptyAfter(row);
        }
    }

    private void fireInsertEmptyBefore(int row) {
        for (MacroTableListener l : tableListeners) {
            l.insertEmptyBefore(row);
        }
    }

    private void firePasteAfter(int row) {
        for (MacroTableListener l : tableListeners) {
            l.pasteAfter(row);
        }
    }

    private void firePasteBefore(int row) {
        for (MacroTableListener l : tableListeners) {
            l.pasteBefore(row);
        }
    }

    private Function[] getFunctions() {
        List<Function> result = new LinkedList<Function>();

        if (model.getAnalogPorts().size() > 0) {
            result.add(new AnalogPortAction());
        }
        result.add(new CriticalFunction());
        result.add(new FlagFunction());
        if (model.getInputPorts().size() > 0) {
            result.add(new InputFunction());
        }
        if (model.getLightPorts().size() > 0) {
            result.add(new LightPortAction());
        }
        result.add(new MacroFunction());
        if (model.getMotorPorts().size() > 0) {
            result.add(new MotorPortAction());
        }
        if (model.getServoPorts().size() > 0) {
            result.add(new ServoPortAction());
        }
        if (model.getSoundPorts().size() > 0) {
            result.add(new SoundPortAction());
        }
        if (model.getSwitchPorts().size() > 0) {
            result.add(new SwitchPortAction());
        }
        result.add(new DelayFunction());
        result.add(new RandomDelayFunction());
        return result.toArray(new Function[0]);
    }

    private Function[] getFunctions(int[] rows) {
        Function[] result = new Function[rows.length];

        for (int index = 0; index < rows.length; index++) {
            result[index] = model.getMacro().getFunction(rows[index]);
        }
        return result;
    }

    private int getRow(Point point) {
        return table.rowAtPoint(point);
    }

    private void handleMouseEvent(MouseEvent e, JPopupMenu popupMenu) {
        if (e.isPopupTrigger()) {
            int row = getRow(e.getPoint());

            if (table.getSelectedRowCount() == 0 && table.getRowCount() > 0 && row >= 0 && row < table.getRowCount()) {
                table.setRowSelectionInterval(row, row);
            }
            table.grabFocus();
            popupMenu.show(e.getComponent(), e.getX(), e.getY());
        }
    }
}
