package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.common.view.panel.DisabledPanel;
import org.bidib.wizard.mvc.main.model.Macro;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.function.Function;
import org.bidib.wizard.mvc.main.model.listener.MacroListener;
import org.bidib.wizard.mvc.main.view.menu.MacroListMenu;
import org.bidib.wizard.mvc.main.view.menu.listener.MacroListMenuListener;
import org.bidib.wizard.mvc.main.view.panel.listener.MacroListListener;
import org.bidib.wizard.mvc.main.view.panel.listener.MacroTableListener;

public class MacroListPanel extends LabelListPanel
    implements MacroListMenuListener, MacroListener, org.bidib.wizard.mvc.main.model.listener.MacroListListener {
    private static final long serialVersionUID = 5506124079986633240L;

    private static final String EMPTY_LABEL = " ";

    private final Collection<MacroListListener> macroListListeners = new LinkedList<MacroListListener>();

    private final MainModel model;

    private final MacroListMenu macroListMenu = new MacroListMenu();

    private final MacroParameterPanel macroParameterPanel;

    private final MacroContentPanel macroContentPanel;

    private final JPanel borderPanel = new JPanel(new GridBagLayout());

    private final TitledBorder border =
        BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), EMPTY_LABEL);

    private final DisabledPanel disabledBorderPanel;

    private Macro macro = null;

    public MacroListPanel(final MainModel model) {
        this.model = model;
        model.addMacroListListener(this);

        setLayout(new GridBagLayout());
        getLabelList().addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                handleMouseEvent(e, macroListMenu);
            }

            public void mouseReleased(MouseEvent e) {
                handleMouseEvent(e, macroListMenu);
            }
        });
        getLabelList().setListData(model.getMacros().toArray());
        getLabelList().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        macroListMenu.addMenuListener(this);

        borderPanel.setBorder(border);

        macroParameterPanel = new MacroParameterPanel(model);

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.fill = GridBagConstraints.BOTH;
        c.gridheight = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.9;
        c.weighty = 0;
        borderPanel.add(macroParameterPanel, c);

        macroContentPanel = new MacroContentPanel(model);
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.gridy++;
        c.weighty = 1;
        borderPanel.add(macroContentPanel, c);

        JPanel panel = new JPanel(new GridBagLayout());

        c.anchor = GridBagConstraints.LINE_START;
        c.gridx = 1;
        panel.add(borderPanel, c);
        disabledBorderPanel = new DisabledPanel(borderPanel);
        panel.add(disabledBorderPanel, c);

        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, new JScrollPane(getLabelList()), panel);

        splitPane.setContinuousLayout(true);
        splitPane.setOneTouchExpandable(true);
        splitPane.setResizeWeight(0.2);

        add(splitPane, c);
        setEnabled(false);

        addButtonPanel(c);
    }

    private void addButtonPanel(GridBagConstraints c) {
        JPanel buttonPanel = new JPanel(new GridBagLayout());

        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.NONE;
        c.gridwidth = 1;
        c.weightx = 0.33;
        c.weighty = 0;

        JButton reloadButton = new JButton(Resources.getString(getClass(), "reload"));

        reloadButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireReloadMacro();
            }
        });
        buttonPanel.add(reloadButton, c);

        c.gridx++;

        JButton transferButton = new JButton(Resources.getString(getClass(), "transfer"));

        transferButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireTransferMacro();
            }
        });
        buttonPanel.add(transferButton, c);

        c.gridx++;

        JButton startButton = new JButton(Resources.getString(getClass(), "start"));

        startButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireStartMacro();
            }
        });
        buttonPanel.add(startButton, c);

        c.anchor = GridBagConstraints.CENTER;
        c.gridwidth = GridBagConstraints.RELATIVE;
        c.gridx++;

        JButton stopButton = new JButton(Resources.getString(getClass(), "stop"));

        stopButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireStopMacro();
            }
        });
        buttonPanel.add(stopButton, c);

        c.anchor = GridBagConstraints.LINE_END;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx++;

        JButton saveButton = new JButton(Resources.getString(getClass(), "save"));

        saveButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireSaveMacro();
            }
        });
        buttonPanel.add(saveButton, c);

        c.anchor = GridBagConstraints.LAST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy++;
        borderPanel.add(buttonPanel, c);
    }

    public void addMacroListListener(MacroListListener l) {
        addLabelChangedListener(l);
        macroListListeners.add(l);
    }

    public void addMacroTableListener(MacroTableListener l) {
        macroContentPanel.addTableListener(l);
    }

    public void setEnabled(boolean enabled) {
        disabledBorderPanel.setEnabled(enabled);
    }

    private void fireExportMacro() {
        for (MacroListListener l : macroListListeners) {
            l.exportMacro(model.getMacro());
        }
    }

    private void fireImportMacro() {
        for (MacroListListener l : macroListListeners) {
            l.importMacro();
        }
    }

    private void fireReloadMacro() {
        for (MacroListListener l : macroListListeners) {
            l.reloadMacro(model.getMacro());
        }
    }

    private void fireSaveMacro() {
        for (MacroListListener l : macroListListeners) {
            l.saveMacro(model.getMacro());
        }
    }

    private void fireStartMacro() {
        for (MacroListListener l : macroListListeners) {
            l.startMacro(model.getMacro());
        }
    }

    private void fireStopMacro() {
        for (MacroListListener l : macroListListeners) {
            l.stopMacro(model.getMacro());
        }
    }

    private void fireInitializeMacro() {
        for (MacroListListener l : macroListListeners) {
            l.initializeMacro(model.getMacro());
        }
    }

    private void fireTransferMacro() {
        for (MacroListListener l : macroListListeners) {
            l.transferMacro(model.getMacro());
        }
    }

    @Override
    public void exportMacro() {
        fireExportMacro();
    }

    @Override
    public void importMacro() {
        fireImportMacro();
    }

    @Override
    public void startMacro() {
        fireStartMacro();
    }

    @Override
    public void stopMacro() {
        fireStopMacro();
    }

    @Override
    public void initializeMacro() {
        fireInitializeMacro();
    }

    @Override
    public void listChanged() {
        getLabelList().setListData(model.getMacros().toArray());
        setEnabled(false);
    }

    @Override
    public void macroChanged() {
        if (macro != null) {
            macro.removeMacroListener(this);
        }

        macro = model.getMacro();

        if (macro != null) {
            macro.addMacroListener(this);
            labelChanged(macro.toString());
        }
        else {
            labelChanged(EMPTY_LABEL);
        }
        setEnabled(macro != null);
    }

    @Override
    public void functionsAdded(int row, Function[] functions) {
    }

    public String getName() {
        return Resources.getString(getClass(), "name");
    }

    @Override
    public void functionRemoved(int row) {
    }

    @Override
    public void functionsRemoved() {
    }

    @Override
    public void labelChanged(String label) {
        if (label != null && !label.equals(EMPTY_LABEL)) {
            label = macro.toString();
        }
        if (label != null && !label.equals(EMPTY_LABEL)) {
            border.setTitle(label + ":");
        }
        else {
            border.setTitle("");
        }
        borderPanel.repaint();
        fireLabelChanged(model.getMacro(), label);
    }
}
