package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JScrollPane;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.ServoPort;
import org.bidib.wizard.mvc.main.model.ServoPortTableModel;
import org.bidib.wizard.mvc.main.model.listener.PortListListener;
import org.bidib.wizard.mvc.main.model.listener.ServoPortListener;
import org.bidib.wizard.mvc.main.view.table.NumberCellEditor;
import org.bidib.wizard.mvc.main.view.table.NumberWithButtonEditor;
import org.bidib.wizard.mvc.main.view.table.NumberWithButtonRenderer;
import org.bidib.wizard.mvc.main.view.table.PortTable;
import org.bidib.wizard.mvc.main.view.table.listener.ButtonListener;

public class ServoPortListPanel extends SimplePortListPanel implements PortListListener {
    private static final long serialVersionUID = 5506124079986633240L;

    private final Collection<ServoPortListener> servoPortListeners = new LinkedList<ServoPortListener>();

    private final MainModel model;

    private final ServoPortTableModel tableModel;

    public ServoPortListPanel(MainModel model) {
        super(null, null, null);
        this.model = model;
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);

        tableModel = new ServoPortTableModel();

        table = new PortTable(
            tableModel, Resources.getString(getClass(), "emptyTable")) {
            private static final long serialVersionUID = 2215982683115274804L;

            public TableCellEditor getCellEditor(int row, int column) {
                TableCellEditor result = null;

                if (column == 1) {
                    result = new NumberCellEditor();
                }
                else if (column >= 2 && column <= 4) {
                    result = new NumberWithButtonEditor(">", column < 4 ? 255 : 100);
                    ((NumberWithButtonEditor) result).addButtonListener(new ButtonListener() {
                        @Override
                        public void buttonPressed(int row, int column) {
                            ServoPort servoPort =
                                (ServoPort) tableModel.getValueAt(row, tableModel.getColumnCount() - 1);

                            if (column == 2) {
                                fireTrimDownButtonPressed(servoPort);
                            }
                            else if (column == 3) {
                                fireTrimUpButtonPressed(servoPort);
                            }
                            else if (column == 4) {
                                fireTestButtonPressed(servoPort);
                            }
                        }
                    });
                }
                else {
                    result = super.getCellEditor(row, column);
                }
                return result;
            }

            public TableCellRenderer getCellRenderer(int row, int column) {
                TableCellRenderer result = null;

                if (column >= 2 && column <= 4) {
                    result = new NumberWithButtonRenderer(">");
                }
                else {
                    result = super.getCellRenderer(row, column);
                }
                return result;
            }
        };
        table.removeColumn(table.getColumnModel().getColumn(table.getColumnCount() - 1));
        table.setRowSelectionAllowed(false);
        add(new JScrollPane(table), c);

        model.addServoPortListListener(this);
    }

    public void addServoPortListener(ServoPortListener l) {
        servoPortListeners.add(l);
        tableModel.addServoPortListener(l);
    }

    private void fireTrimDownButtonPressed(ServoPort servoPort) {
        for (ServoPortListener l : servoPortListeners) {
            l.trimDownButtonPressed(servoPort);
        }
    }

    private void fireTrimUpButtonPressed(ServoPort servoPort) {
        for (ServoPortListener l : servoPortListeners) {
            l.trimUpButtonPressed(servoPort);
        }
    }

    private void fireTestButtonPressed(ServoPort servoPort) {
        for (ServoPortListener l : servoPortListeners) {
            l.testButtonPressed(servoPort);
        }
    }

    @Override
    public void listChanged() {
        tableModel.setRowCount(0);
        for (ServoPort servoPort : model.getServoPorts()) {
            tableModel.addRow(servoPort);
        }
    }
}
