package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Collection;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.Port;
import org.bidib.wizard.mvc.main.model.SimplePortTableModel;
import org.bidib.wizard.mvc.main.model.listener.OutputListener;
import org.bidib.wizard.mvc.main.model.listener.PortListListener;
import org.bidib.wizard.mvc.main.view.table.MyTable;
import org.bidib.wizard.mvc.main.view.table.PortTable;

public abstract class SimplePortListPanel extends JPanel implements PortListListener {
    private static final long serialVersionUID = 862441722564650906L;

    protected final SimplePortTableModel tableModel;

    protected final Collection<? extends Port> ports;

    protected MyTable table = null;

    public SimplePortListPanel(final SimplePortTableModel tableModel, Collection<? extends Port> ports,
        String emptyTableText) {
        this.tableModel = tableModel;
        this.ports = ports;
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);

        table = new PortTable(tableModel, emptyTableText);
        if (table.getColumnCount() > 0) {
            table.removeColumn(table.getColumnModel().getColumn(table.getColumnCount() - 1));
        }
        table.setRowSelectionAllowed(false);
        add(new JScrollPane(table), c);
    }

    public void addPortListener(OutputListener l) {
        tableModel.addPortListener(l);
    }

    public String getName() {
        return Resources.getString(getClass(), "name");
    }

    @Override
    public void listChanged() {
        tableModel.setRowCount(0);
        for (Port output : ports) {
            tableModel.addRow(output);
        }
        if (table.getColumnCount() > 1) {
            table.packColumn(table.getColumnCount() - 1, 2);
        }
    }
}
