package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Date;

import javax.swing.JPanel;

import org.bidib.wizard.mvc.common.view.panel.DisabledPanel;
import org.bidib.wizard.mvc.main.model.StatusModel;
import org.bidib.wizard.mvc.main.view.panel.listener.StatusListener;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.model.listener.PreferencesListener;

public class StatusPanel extends JPanel {
    private static final long serialVersionUID = 6259235698654221621L;

    private final StatusButtonPanel statusButtonPanel;

    public StatusPanel(StatusModel model) {
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.2;
        c.weighty = 1;
        statusButtonPanel = new StatusButtonPanel(model);
        add(statusButtonPanel, c);

        c.anchor = GridBagConstraints.CENTER;
        c.gridx++;
        c.weightx = 0.7;

        final DisabledPanel disabledLedPanel = new DisabledPanel(new LedPanel(model));

        disabledLedPanel.setEnabled(!Preferences.getInstance().isMock());
        add(disabledLedPanel, c);

        c.anchor = GridBagConstraints.LINE_END;
        c.gridx++;
        c.weightx = 0.1;
        add(new TimePanel(model), c);
        Preferences.getInstance().addPreferencesListener(new PreferencesListener() {
            @Override
            public void commPortChanged(String commPort) {
            }

            @Override
            public void mockChanged(boolean mock) {
                disabledLedPanel.setEnabled(!mock);
            }

            @Override
            public void startTimeChanged(Date startTime) {
            }

            @Override
            public void timeFactorChanged(int timeFactor) {
            }
        });
    }

    public void addStatusListener(StatusListener l) {
        statusButtonPanel.addStatusListener(l);
    }
}
