/*
 Copyright 2009 umbalaconmeogia

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
package org.bidib.wizard.mvc.main.view.statusbar;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import org.bidib.wizard.comm.BoosterStatus;
import org.bidib.wizard.darrylbu.util.SwingUtils;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.listener.BoosterStatusListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class StatusBar extends JLabel {
    private static final long serialVersionUID = 1L;

    private static final Logger LOGGER = LoggerFactory.getLogger(StatusBar.class);

    /**
     * Constructor.
     */
    public StatusBar(final MainModel model) {
        super(" ");

        // Set border.
        setBorder(BorderFactory.createLoweredBevelBorder());

        // Set the text style to be plain.
        SwingUtils.setFontBold(this, false);

        model.addBoosterStatusListener(new BoosterStatusListener() {

            @Override
            public void voltageChanged(int voltage) {
                // TODO Auto-generated method stub

            }

            @Override
            public void temperatureChanged(int temperature) {
                // TODO Auto-generated method stub

            }

            @Override
            public void stateChanged(BoosterStatus status) {
                // TODO Auto-generated method stub

            }

            @Override
            public void maximumCurrentChanged(int maximumCurrent) {
                // TODO Auto-generated method stub

            }

            @Override
            public void currentChanged(final int current) {
                if (SwingUtilities.isEventDispatchThread()) {
                    internalCurrentChanged(current);
                }
                else {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            internalCurrentChanged(current);
                        }
                    });
                }
            }

            private void internalCurrentChanged(final int current) {
                LOGGER.debug("Current has changed: {}", current);
                if (current > 4000) {
                    StringBuffer sb = new StringBuffer("Received high current with value: ");
                    sb.append(current);
                    sb.append("mA");
                    LOGGER.warn(sb.toString());
                    setStatus(sb.toString(), 30);
                }
            }
        });
    }

    /**
     * Display a message on the status bar.
     * @param message The message to be displayed.
     */
    public void setStatus(String message) {
        LOGGER.debug("Set status message: {}", message);

        setStatus(message, -1);
    }

    /**
     * Display a message on the status bar then clear it after specified time.
     * @param message The message to be displayed.
     * @param seconds Time wait for clearing the message (in seconds).
     * 					Any value lesser than 1 disable this functionality.
     */
    public synchronized void setStatus(String message, int seconds) {
        if (seconds > -1) {
            setForeground(Color.RED);
        }
        else {
            setForeground(Color.BLACK);
        }
        setText(" " + message);
        if (seconds > 0) {
            if (cleanTimer != null) {
                LOGGER.info("Stop the clean timer.");
                cleanTimer.stop();
                cleanTimer = null;
            }
            if (cleanTimer == null) {
                cleanTimer = new Timer(seconds * 1000, new ActionListener() {

                    @Override
                    public void actionPerformed(ActionEvent e) {
                        LOGGER.info("Reset the statusbar.");
                        setForeground(Color.BLACK);
                        setText(" ");
                    }
                });
                cleanTimer.setRepeats(false);
                cleanTimer.start();
            }
        }
    }

    private Timer cleanTimer;
}
