package org.bidib.wizard.mvc.main.view.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import org.bidib.wizard.comm.AnalogPortStatus;
import org.bidib.wizard.comm.BidibStatus;
import org.bidib.wizard.comm.CriticalFunctionStatus;
import org.bidib.wizard.comm.FlagStatus;
import org.bidib.wizard.comm.InputStatus;
import org.bidib.wizard.comm.LightPortStatus;
import org.bidib.wizard.comm.MacroStatus;
import org.bidib.wizard.comm.MotorPortStatus;
import org.bidib.wizard.comm.ServoPortStatus;
import org.bidib.wizard.comm.SoundPortStatus;
import org.bidib.wizard.comm.SwitchPortStatus;
import org.bidib.wizard.dialog.LabelDialog;
import org.bidib.wizard.mvc.main.view.menu.PortListMenu;
import org.bidib.wizard.mvc.main.view.menu.listener.LabelListMenuListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MyTable extends JTable {
    private static final Logger LOGGER = LoggerFactory.getLogger(MyTable.class);

    private static final long serialVersionUID = -7751411934378167130L;

    private final String emptyTableText;

    private final PortListMenu menu = new PortListMenu();

    private MouseEvent popupEvent = null;

    public MyTable(TableModel dm, String emptyTableText) {
        super(dm);
        this.emptyTableText = emptyTableText;
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                handleMouseEvent(e, menu);
            }

            public void mouseReleased(MouseEvent e) {
                handleMouseEvent(e, menu);
            }
        });
        menu.addMenuListener(new LabelListMenuListener() {
            @Override
            public void editLabel() {
                final int row = getRow(popupEvent.getPoint());
                if (row > -1) {
                    final Object value = getValueAt(row, 0);

                    if (value instanceof String) {
                        new LabelDialog(
                            (String) value, popupEvent.getXOnScreen(), popupEvent.getYOnScreen()) {
                            @Override
                            public void labelChanged(String label) {
                                setValueAt(label, row, 0);
                            }
                        };
                    }
                }
                else {
                    LOGGER.warn("The row is not available!");
                }
            }
        });
        setRowHeight(getRowHeight() + 4);
    }

    public Object[] getActions(BidibStatus action) {
        List<Object> result = new LinkedList<Object>();

        if (action instanceof AnalogPortStatus) {
            for (AnalogPortStatus actionEnum : AnalogPortStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof CriticalFunctionStatus) {
            for (CriticalFunctionStatus actionEnum : CriticalFunctionStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof FlagStatus) {
            for (FlagStatus actionEnum : FlagStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof InputStatus) {
            for (InputStatus actionEnum : InputStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof LightPortStatus) {
            for (LightPortStatus actionEnum : LightPortStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof MacroStatus) {
            for (MacroStatus actionEnum : MacroStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof MotorPortStatus) {
            for (MotorPortStatus actionEnum : MotorPortStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof ServoPortStatus) {
            for (ServoPortStatus actionEnum : ServoPortStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof SoundPortStatus) {
            for (SoundPortStatus actionEnum : SoundPortStatus.values()) {
                result.add(actionEnum);
            }
        }
        else if (action instanceof SwitchPortStatus) {
            for (SwitchPortStatus actionEnum : SwitchPortStatus.values()) {
                result.add(actionEnum);
            }
        }
        return result.toArray(new Object[0]);
    }

    private int getColumn(Point point) {
        return columnAtPoint(point);
    }

    private int getRow(Point point) {
        return rowAtPoint(point);
    }

    private void handleMouseEvent(MouseEvent e, JPopupMenu popupMenu) {
        if (e.isPopupTrigger() && getRowCount() > 0) {
            popupEvent = e;

            int row = getRow(e.getPoint());
            int column = getColumn(e.getPoint());

            if (row >= 0 && column == 0 && getValueAt(row, column) instanceof String) {
                if (row >= 0 && getSelectedRowCount() == 0) {
                    setRowSelectionInterval(row, row);
                }
                grabFocus();
                popupMenu.show(e.getComponent(), e.getX(), e.getY());
            }
        }
    }

    public Dimension getPreferredScrollableViewportSize() {
        return getPreferredSize();
    }

    public boolean getScrollableTracksViewportHeight() {
        return getPreferredSize().height < getParent().getHeight();
    }

    public void packColumn(int vColIndex, int margin) {
        DefaultTableColumnModel colModel = (DefaultTableColumnModel) getColumnModel();
        TableColumn col = colModel.getColumn(vColIndex);
        TableCellRenderer renderer = col.getHeaderRenderer();
        int width = 0;

        if (renderer == null && getTableHeader() != null) {
            renderer = getTableHeader().getDefaultRenderer();
        }
        if (renderer != null) {
            Component comp = renderer.getTableCellRendererComponent(this, col.getHeaderValue(), false, false, 0, 0);

            width = comp.getPreferredSize().width;
        }

        for (int r = 0; r < getRowCount(); r++) {
            renderer = getCellRenderer(r, vColIndex);
            Component comp =
                renderer.getTableCellRendererComponent(this, getValueAt(r, vColIndex), false, false, r, vColIndex);

            width = Math.max(width, comp.getPreferredSize().width);
        }
        width += 2 * margin;
        col.setMaxWidth(width);
        col.setPreferredWidth(width);
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (emptyTableText != null && getRowCount() == 0) {
            Graphics2D g2d = (Graphics2D) g;

            g2d.setColor(Color.BLACK);
            g2d.drawString(emptyTableText, 10, 20);
        }
    }
}
