package org.bidib.wizard.mvc.main.view.table;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.DecimalFormat;

import javax.swing.AbstractCellEditor;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellEditor;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

public class NumberWithLabelEditor extends AbstractCellEditor implements TableCellEditor {
    private static final long serialVersionUID = -5074223597210117377L;

    private final JPanel panel = new JPanel();

    private final JFormattedTextField textField = new JFormattedTextField(3);

    private final NumberFormatter formatter = new NumberFormatter(
        new DecimalFormat("#")) {
        private static final long serialVersionUID = -1986728550578313511L;

        {
            setAllowsInvalid(false);
            setMinimum(0);
        }
    };

    private final JLabel leftLabel;

    private final JLabel rightLabel;

    public NumberWithLabelEditor(String leftText, String rightText, int maximum) {
        formatter.setMaximum(maximum);
        textField.setFormatterFactory(new DefaultFormatterFactory(formatter, formatter, formatter));
        textField.setHorizontalAlignment(SwingConstants.RIGHT);
        leftLabel = new JLabel(leftText);
        rightLabel = new JLabel(rightText);
        panel.setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.insets = new Insets(0, 0, 2, 2);
        c.weightx = 0;
        panel.add(leftLabel, c);
        c.gridx++;
        c.weightx = leftText.isEmpty() ? 0.5 : 1;
        panel.add(textField, c);
        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.gridx++;
        c.weightx = leftText.isEmpty() ? 0.5 : 0;
        panel.add(rightLabel, c);
    }

    public NumberWithLabelEditor(String leftText, String rightText) {
        this(leftText, rightText, 255);
    }

    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
        if (isSelected) {
            panel.setForeground(table.getSelectionForeground());
            panel.setBackground(table.getSelectionBackground());
        }
        else {
            panel.setForeground(table.getForeground());
            panel.setBackground(table.getBackground());
        }
        textField.setText(value != null ? value.toString() : "");
        return panel;
    }

    @Override
    public Object getCellEditorValue() {
        return textField.getText();
    }

    @Override
    public boolean stopCellEditing() {
        try {
            int v = Integer.valueOf(textField.getText());
            if (v < 0) {
                throw new NumberFormatException();
            }
        }
        catch (NumberFormatException e) {
            return false;
        }
        return super.stopCellEditing();
    }
}
