package org.bidib.wizard.mvc.preferences.controller;

import gnu.io.CommPortIdentifier;

import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.List;

import javax.swing.JFrame;

import org.bidib.jbidibc.Bidib;
import org.bidib.wizard.mvc.preferences.model.CommPort;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.model.PreferencesModel;
import org.bidib.wizard.mvc.preferences.view.PreferencesView;
import org.bidib.wizard.mvc.preferences.view.listener.PreferencesViewListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PreferencesController {
    private static final Logger LOGGER = LoggerFactory.getLogger(PreferencesController.class);

    private final JFrame parent;

    private PreferencesModel model;

    public PreferencesController(JFrame parent) {
        this.parent = parent;
    }

    private void loadCommPorts() {
        if (!model.isMock()) {
            LOGGER.info("Load the comm ports, model: {}", model);
            Collection<CommPort> commPorts = new HashSet<CommPort>();

            // use Bidib because we must load the RXTX libraries
            List<CommPortIdentifier> portIdentifiers = Bidib.getInstance().getPortIdentifiers();

            if (portIdentifiers != null) {
                for (CommPortIdentifier id : portIdentifiers) {
                    // only add serial ports
                    if (id.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                        commPorts.add(new CommPort(id.getName()));
                    }
                }
            }
            // check if the currently configured COM port is added already, if not add it
            //            String configuredPort = Preferences.getInstance().getCommPort();
            //            if (!StringUtils.isBlank(configuredPort)) {
            //                CommPort commPort = new CommPort(configuredPort);
            //                if (!commPorts.contains(commPort)) {
            //                    // add port
            //                    commPorts.add(commPort);
            //                }
            //            }

            model.setCommPorts(commPorts);
        }
    }

    public void start() {
        LOGGER.debug("Start the preferences controller.");
        model = Preferences.getInstance().createPreferencesModel();

        loadCommPorts();

        PreferencesView view = new PreferencesView(parent, model);

        view.addPreferencesViewListener(new PreferencesViewListener() {
            @Override
            public void commPortChanged(String port) {
                LOGGER.info("The comm port has changed: {}", port);

                // set the new commPort in the model
                model.setCommPort(port);
            }

            @Override
            public void mockChanged(boolean mock) {
                LOGGER.debug("Mock has changed: {}", mock);

                // set the new mock value in the model
                model.setMock(mock);

                LOGGER.debug("Reload the comm ports.");
                loadCommPorts();
            }

            @Override
            public void save() {
                // save the mode in the preferences
                Preferences.getInstance().save(model);

                // reload the logger configuration because the path to the log file might have changed 
                Preferences.getInstance().reloadLoggerConfiguration(false);
            }

            @Override
            public void cancel() {
            }

            @Override
            public void startTimeChanged(Date startTime) {
                // set the new start time in the model
                model.setStartTime(startTime);
            }

            @Override
            public void timeFactorChanged(int timeFactor) {
                // set the new time factor in the model
                model.setTimeFactor(timeFactor);
            }
        });
    }
}
