package org.bidib.wizard.mvc.preferences.model;

import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;

import org.bidib.wizard.mvc.preferences.model.listener.PreferencesModelListener;

public class PreferencesModel {
    private final Collection<PreferencesModelListener> listeners = new LinkedList<PreferencesModelListener>();

    private final Collection<CommPort> commPorts = new HashSet<CommPort>();

    private String port;

    private boolean mock;

    private Date startTime;

    private int timeFactor;

    private String logFilePath;

    private String labelPath;

    public void addPreferencesModelListener(PreferencesModelListener l) {
        listeners.add(l);
    }

    private void fireCommPortsChanged() {
        for (PreferencesModelListener l : listeners) {
            l.commPortsChanged();
        }
    }

    public Collection<CommPort> getCommPorts() {
        return commPorts;
    }

    /**
     * Set the commPorts and notify the listeners
     * @param commPorts the commPorts to set
     */
    public void setCommPorts(Collection<CommPort> commPorts) {
        this.commPorts.clear();
        this.commPorts.addAll(commPorts);
        fireCommPortsChanged();
    }

    /**
     * @return the port
     */
    public String getCommPort() {
        return port;
    }

    /**
     * @param port the port to set
     */
    public void setCommPort(String port) {
        this.port = port;
    }

    /**
     * @param port the port to set
     */
    public void setCommPort(CommPort port) {
        if (port != null && port.getName() != null) {
            this.port = port.getName();
        }
        else {
            this.port = "";
        }
    }

    /**
     * @return the mock
     */
    public boolean isMock() {
        return mock;
    }

    /**
     * @param mock the mock to set
     */
    public void setMock(boolean mock) {
        this.mock = mock;
    }

    /**
     * @return the startTime
     */
    public Date getStartTime() {
        return startTime;
    }

    /**
     * @param startTime the startTime to set
     */
    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    /**
     * @return the timeFactor
     */
    public int getTimeFactor() {
        return timeFactor;
    }

    /**
     * @param timeFactor the timeFactor to set
     */
    public void setTimeFactor(int timeFactor) {
        this.timeFactor = timeFactor;
    }

    /**
     * @return the logFilePath
     */
    public String getLogFilePath() {
        return logFilePath;
    }

    /**
     * @param logFilePath the logFilePath to set
     */
    public void setLogFilePath(String logFilePath) {
        this.logFilePath = logFilePath;
    }

    /**
     * @return the labelPath
     */
    public String getLabelPath() {
        return labelPath;
    }

    /**
     * @param labelPath the labelPath to set
     */
    public void setLabelPath(String labelPath) {
        this.labelPath = labelPath;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("PreferencesModel, port: ");
        sb.append(getCommPort()).append(", mock: ");
        sb.append(isMock()).append(", startTime: ");
        sb.append(getStartTime()).append(", timeFactor: ");
        sb.append(getTimeFactor()).append(", logFilePath: ");
        sb.append(getLogFilePath()).append(", labelPath: ");
        sb.append(getLabelPath());

        return sb.toString();
    }
}
